#!/usr/bin/env python

import xml.dom.minidom
import xml.xpath
import xml.xpath.Context
from xml.sax.saxutils import unescape
import sys

class DocStringGetter:
    functionXPath = '//root/function[@name="%s"]'
    descriptionXPath = functionXPath + '/description'
    parameterXPath = functionXPath + '/parameters/parameter'
    classXPath = '//root/class[@name="%s"]'
    classDescXPath = classXPath + '/description'

    def __init__(self, xmlFile):
        document = xml.dom.minidom.parse(xmlFile)
        self.context = xml.xpath.Context.Context(document.documentElement)
        self.fileName = xmlFile

    @staticmethod
    def _makeCString(s):
        rep = repr(unescape(s)).strip("'\"u")
        for before,after in [('"', '\\"'), ('\n', '\\n'), ('@', ''), ('#', '')]:
            rep = rep.replace(before,after)
        return '"%s"' % rep
       

    def getOriginalDocParameterList(self, functionName):
        element = xml.xpath.Evaluate(self.parameterXPath % functionName,
                                     context=self.context)
        return map(lambda x: x.getAttribute("name"), element)

    @staticmethod
    def getParameterList(meth):
        """Returns a list of strings containing the names of the parameters of
        functionName"""
        return map(lambda x: x.pname, meth.params)

    def getPrototype(self, meth, functionName):
        param_list = DocStringGetter.getParameterList(meth)
        original_param_list = self.getOriginalDocParameterList(functionName)

        if len(original_param_list) > len(param_list):
            object_prefix = "%s." % original_param_list[0]
        else:
            object_prefix = ""

        if meth.ret and (meth.ret != "none"):
            # yeah, that replace is quite a hack...
            ret_suffix = " -> %s" % meth.ret.replace("Pgm", "pgm.")
        else:
            ret_suffix = ""

        return "%s%s(%s)%s\n" % (object_prefix, meth.name, \
                         ", ".join(DocStringGetter.getParameterList(meth)), \
                         ret_suffix)

    def _getContentFromXPath(self, xpath, name):
        element = xml.xpath.Evaluate(xpath % name, context=self.context)
        try:
            doc_string = "\n" + element[0].firstChild.data.strip()
        except IndexError:
            print >> sys.stderr, "The entity %s is not documented in %s" % (
                                                  name, self.fileName)
            doc_string = ""
        return doc_string

    def getFunctionDocString(self, meth, functionName):
        doc_string = \
                 self._getContentFromXPath(self.descriptionXPath, functionName)

        prototype = self.getPrototype(meth, functionName)

        return DocStringGetter._makeCString( prototype + doc_string)

    def getClassDocString(self, name):
        doc_string = self._getContentFromXPath(self.classDescXPath, name)

        return DocStringGetter._makeCString(doc_string)




# -*- mode: python; coding: utf-8 -*-
#
# Pigment Python tools
#
# Copyright © 2006, 2007, 2008 Fluendo Embedded S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

import pgm

class Drawable(object):

    def __init__(self):
        object.__init__(self)
        self._position_offset=(0.0, 0.0, 0.0)
        self._opacity=self.widget_class.get_opacity(self)
        self._opacity_factor=1.0
        self._visible=self.widget_class.is_visible (self)
        self._parent_visibility=True

    def position__get(self):
        absolute_position = self.widget_class.get_position(self)
        x = absolute_position[0]-self._position_offset[0]
        y = absolute_position[1]-self._position_offset[1]
        z = absolute_position[2]-self._position_offset[2]
        return (x, y, z)


    def x__get(self):
        x = self.widget_class.get_position(self)[0]
        relative_x = x - self._position_offset[0]
        return relative_x


    def x__set(self, value):
        #FIXME : there is not set_x function in pigment.... need to add
        current_position = self.position
        current_position = (value, current_position[1], current_position[2])
        self.position = current_position


    def y__get(self):
        y = self.widget_class.get_position(self)[1]
        relative_y = y - self._position_offset[1]
        return relative_y


    def y__set(self, value):
        #FIXME : there is not set_x function in pigment.... need to add
        current_position = self.position
        current_position = (current_position[0], value, current_position[2])
        self.position = current_position


    def z__get(self):
        z = self.widget_class.get_position(self)[2]
        relative_z = z - self._position_offset[2]
        return relative_z


    def z__set(self, value):
        #FIXME : there is not set_x function in pigment.... need to add
        current_position = self.position
        current_position = (current_position[0], current_position[1], value)
        self.position = current_position


    def position__set(self, relative_position):
        x = self._position_offset[0]+relative_position[0]
        y = self._position_offset[1]+relative_position[1]
        z = self._position_offset[2]+relative_position[2]
        self.widget_class.set_position(self, x, y, z)


    def position_offset__get(self):
        return self._position_offset


    def position_offset__set(self, value):
        previous_position = self.position
        self._position_offset = value
        self.position = previous_position

    def opacity_factor_offset__set (self, factor):
        self._opacity_factor = factor
        self.opacity = self._opacity

    def x_offset__get(self):
        return self._position_offset[0]


    def y_offset__get(self):
        return self._position_offset[1]


    def z_offset__get(self):
        return self._position_offset[2]


    def x_offset__set(self, value):
        previous_position = self.position
        new_position = (value, previous_position[1], previous_position[2])
        self.position_offset = new_position


    def y_offset__set(self, value):
        previous_position = self.position
        new_position = (previous_position[0], value, previous_position[2])
        self.position_offset = new_position


    def z_offset__set(self, value):
        previous_position = self.position
        new_position = (previous_position[0], previous_position[1], value)
        self.position_offset = new_position

    def opacity__get(self):
        return self._opacity

    def opacity__set(self, value):
        self._opacity = value
        new_opacity = value * self._opacity_factor
        self.widget_class.set_opacity(self, int (new_opacity))

    def parent_visibility__get (self):
        return self._parent_visibility

    def parent_visibility__set (self, value):
        self._parent_visibility = value
        self.visible=self._visible

    def visible__get (self):
        return self.widget_class.is_visible(self)

    def visible__set(self, value):
        self._visible = value
        if self._visible and self._parent_visibility:
            self.widget_class.show(self)
        else:
            self.widget_class.hide(self)

    position = property(position__get, position__set)
    position_offset = property(position_offset__get, position_offset__set)
    x_offset = property(x_offset__get, x_offset__set)
    y_offset = property(y_offset__get, y_offset__set)
    z_offset = property(z_offset__get, z_offset__set)
    opacity = property(opacity__get, opacity__set)
    visible = property(visible__get, visible__set)
    parent_visibility = property(parent_visibility__get, parent_visibility__set)
    x = property(x__get, x__set)
    y = property(y__get, y__set)
    z = property(z__get, z__set)



# -*- mode: python; coding: utf-8 -*-
#
# Pigment Python binding cloning example
#
# Copyright © 2006, 2007, 2008 Fluendo Embedded S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# Author: Loïc Molinari <loic@fluendo.com>

import sys, pgm, gobject, gst
from pgm.timing import controller, modifier, keyframe

# Terminate the mainloop on a delete event
def on_delete(viewport, event, pipeline):
    pipeline.set_state(gst.STATE_NULL)
    pgm.main_quit()

# Terminate the mainloop on 'Escape' or 'q' key press event
def on_key_press(viewport, event, pipeline):
    if event.keyval == pgm.keysyms.q or event.keyval == pgm.keysyms.Escape:
        pgm.main_quit()

def main(args):
    # Argument check
    if len(args) < 1:
        print 'usage: %s uri' % sys.argv[0]
        return -1

    # OpenGL viewport creation
    gl = pgm.viewport_factory_make('opengl')
    gl.title = 'Video cloning'

    # Canvas and image drawables creation
    cvs = pgm.Canvas()
    img1 = pgm.Image()
    img2 = pgm.Image()
    img3 = pgm.Image()

    # Bind the canvas to the OpenGL viewport
    gl.set_canvas(cvs)

    # Images properties
    img1.position = (0.1, 0.0, 0.0)
    img1.size = (1.8, 2.0)
    img1.fg_color = (255, 100, 100, 255)
    img1.bg_a = 0
    img2.position = (1.1, 0.5, 0.0)
    img2.size = (1.8, 2.0)
    img2.fg_color = (100, 255, 100, 155)
    img2.bg_a = 0
    img3.position = (2.0, 1.0, 0.0)
    img3.size = (1.8, 2.0)
    img3.fg_color = (100, 100, 255, 255)
    img3.bg_a = 0

    # Drawables need to be shown
    img1.show()
    img2.show()
    img3.show()

    # Add them to the middle layer of the canvas
    cvs.add(pgm.DRAWABLE_MIDDLE, img1, img3, img2)

    # GStreamer pipeline setup
    sink = gst.element_factory_make('pgmimagesink')
    pipeline = gst.element_factory_make('playbin')
    pipeline.set_property('uri', sys.argv[1])
    pipeline.set_property('video-sink', sink)
    sink.set_property('image', img1)
    pipeline.set_state(gst.STATE_PLAYING)

    # Cloning from the first image
    img2.set_from_image(img1)
    img3.set_from_image(img1)

    # Create a animation position timeline
    tml = [keyframe.KeyFrame(1.0, 0.0),
           keyframe.KeyFrame(0.0, 1.0)]

    # Create the animation modifiers
    mdf1 = modifier.Modifier([img1], 'y', tml)
    mdf2 = modifier.Modifier([img2], 'y', tml)
    mdf3 = modifier.Modifier([img3], 'y', tml)

    # Create the animation controllers
    ctrl1 = controller.Controller(duration=700, resolution=7,
                                 repeat_behavior=controller.REVERSE,
                                 repeat_count=controller.INFINITE,
                                 transformation=controller.DECELERATE,
                                 modifiers=[mdf1])
    ctrl2 = controller.Controller(duration=800, resolution=7,
                                 repeat_behavior=controller.REVERSE,
                                 repeat_count=controller.INFINITE,
                                 transformation=controller.DECELERATE,
                                 modifiers=[mdf2])
    ctrl3 = controller.Controller(duration=900, resolution=7,
                                 repeat_behavior=controller.REVERSE,
                                 repeat_count=controller.INFINITE,
                                 transformation=controller.DECELERATE,
                                 modifiers=[mdf3])

    # Start the controllers
    ctrl1.start()
    ctrl2.start()
    ctrl3.start()

    # Let's start the mainloop
    gl.connect('delete-event', on_delete, pipeline)
    gl.connect('key-press-event', on_key_press, pipeline)
    gl.show()
    pgm.main()

if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



# -*- mode: python; coding: utf-8 -*-
#
# Pigment Python tools
#
# Copyright © 2006, 2007, 2008 Fluendo Embedded S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

"""
Pigment utility module
"""

import math

PI, PI2 = math.pi, math.pi/2.0


def clamp(x, low, high):
    """Clamp a value in the range [low, high].
    """
    return max(low, min(high, x))


def lerp(a, b, factor):
    """Linear interpolation applied on a number.
    """
    return clamp(factor, 0.0, 1.0) * (b - a) + a

def serp(a, b, factor):
    """Sinusoidal interpolation applied on a number
    """
    return math.sin(clamp(factor, 0.0, 1.0)*PI2) * (b - a) + a

def lerp_seq(a, b, factor):
    """Linear interpolation applied on sequence like list or tuple.
    """
    diff = map(lambda x,y: x-y,  b, a)
    mult = map(lambda x: clamp(factor, 0.0, 1.0)*x, diff)
    return tuple (map(lambda x,y: x+y, mult, a))



# -*- mode: python; coding: utf-8 -*-
#
# Pigment Python tools
#
# Copyright © 2006, 2007, 2008 Fluendo Embedded S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

"""
"""

from pgm.utils import classinit, maths


class KeyFrame(object):
    """
    """

    # Allows property fget/fset/fdel/doc overriding
    __metaclass__ = classinit.ClassInitMeta
    __classinit__ = classinit.build_properties

    def __init__(self, value=None, time=0.0):
        self.value = value
        self.time = time

    def __repr__(self):
        return "<keyframe.KeyFrame(%r, %.2f)>" % (self._value, self._time)

    # Property definitions

    def value__get(self):
        """The value"""
        return self._value

    def value__set(self, value):
        self._value = value

    def time__get(self):
        """The time in the range [0.0, 1.0]"""
        return self._time

    def time__set(self, time):
        self._time = maths.clamp(time, 0.0, 1.0)



# -*- mode: python; coding: utf-8 -*-
#
# Pigment Python binding groups example
#
# Copyright © 2006, 2007, 2008 Fluendo Embedded S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# Author: Florian Boucault <florian@fluendo.com>

import sys, pgm
from pgm.graph import group, image, text

# Handles the key presses
def on_key_press(viewport, event, step_size, parent_group, child_group):
    if event.keyval == pgm.keysyms.q or event.keyval == pgm.keysyms.Escape:
        pgm.main_quit()

    elif event.keyval == pgm.keysyms.Right:
        parent_group.x += step_size

    elif event.keyval == pgm.keysyms.Left:
        parent_group.x -= step_size

    elif event.keyval == pgm.keysyms.Down:
        parent_group.y += step_size

    elif event.keyval == pgm.keysyms.Up:
        parent_group.y -= step_size

    elif event.keyval == pgm.keysyms.g:
        child_group.x += step_size

    elif event.keyval == pgm.keysyms.d:
        child_group.x -= step_size

    elif event.keyval == pgm.keysyms.f:
        child_group.y += step_size

    elif event.keyval == pgm.keysyms.r:
        child_group.y -= step_size

    elif event.keyval == pgm.keysyms.e:
        parent_group.remove(child_group)

    elif event.keyval == pgm.keysyms.t:
        parent_group.add(child_group)

    elif event.keyval == pgm.keysyms.y:
        sx, sy = parent_group.size
        new_sx = sx + 0.1
        new_sy = sy + 0.1
        parent_group.size = (new_sx, new_sy)

    elif event.keyval == pgm.keysyms.h:
        sx, sy = parent_group.size
        new_sx = sx - 0.1
        new_sy = sy - 0.1
        parent_group.size = (new_sx, new_sy)

# Terminate the mainloop on a delete event
def on_delete(viewport, event):
    pgm.main_quit()


def main(args):
    # OpenGL viewport creation
    gl = pgm.viewport_factory_make('opengl')
    gl.title = 'Wrapping drawables'

    # Canvas creation
    cvs = pgm.Canvas()

    # Bind the canvas to the OpenGL viewport
    gl.set_canvas(cvs)


    # Create text drawables
    txt1 = text.Text()
    txt1.label = "root text 1"
    txt1.height = 0.15
    txt1.size = (1.5, 0.75)
    txt1.fg_color = (255, 0, 0, 255)
    txt1.bg_color = (0, 0, 0, 255)
    txt1.visible = True
    cvs.add(pgm.DRAWABLE_MIDDLE, txt1)

    txt2 = text.Text()
    txt2.label = "root text 2"
    txt2.height = 0.25
    txt2.size = (1.5, 0.75)
    txt2.fg_color = (0, 255, 0, 255)
    txt2.bg_color = (0, 0, 0, 0)
    txt2.visible = True
    cvs.add(pgm.DRAWABLE_MIDDLE, txt2)

    txt3 = text.Text()
    txt3.label = "child text 1"
    txt3.height = 0.1
    txt3.size = (1.5, 0.75)
    txt3.fg_color = (0, 0, 255, 255)
    txt3.bg_color = (0, 0, 0, 0)
    txt3.visible = True
    cvs.add(pgm.DRAWABLE_MIDDLE, txt3)

    txt4 = text.Text()
    txt4.label = "child text 2"
    txt4.height = 0.2
    txt4.size = (1.5, 0.75)
    txt4.fg_color = (255, 0, 255, 255)
    txt4.bg_color = (0, 0, 0, 0)
    txt4.visible = True
    cvs.add(pgm.DRAWABLE_MIDDLE, txt4)

    # Create an image drawable
    img1 = image.Image()
    img1.set_from_file("examples/pictures/fluendo.png")
    img1.size = (1.0, 1.0)
    img1.bg_color = (0, 0, 0, 0)
    img1.visible = True
    cvs.add(pgm.DRAWABLE_MIDDLE, img1)


    # Create two groups, one containing the other
    child_group = group.Group(cvs, pgm.DRAWABLE_MIDDLE)
    child_group.visible = True
    child_group.position = (2.0, 1.0, 0.0)

    child_group.add(txt1)
    child_group.add(txt2)

    parent_group = group.Group(cvs, pgm.DRAWABLE_MIDDLE)
    parent_group.visible = True
    parent_group.position = (0.0, 0.0, 0.0)

    parent_group.add(txt3)
    parent_group.add(txt4)
    parent_group.add(img1)
    parent_group.add(child_group)


    # Let's start the mainloop
    gl.connect('delete-event', on_delete)
    gl.connect('key-press-event', on_key_press, 0.5, parent_group, child_group)
    gl.show()
    pgm.main()

if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



# -*- mode: python; coding: utf-8 -*-
#
# Pigment Python binding
#
# Copyright © 2006, 2007, 2008 Fluendo Embedded S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# Author(s): Loïc Molinari <loic@fluendo.com>
#            Alessandro Decina <alessandro@fluendo.com>
#
# The LazyModule class comes from the Python GTK+ binding.
# Copyright © 2006 Johan Dahlin

from _pgm import *
import sys

__version__ = version()

class LazyModule(object):
    def __init__(self, name, locals):
        self._name = name
        self._locals = locals
        self._modname = '%s.%s' % (self._locals.get('__name__'), self._name)

    def __getattr__(self, attr):
        module = __import__(self._name, self._locals, {}, ' ')
        sys.modules[self._modname] = module
        if attr == '__members__':
            return dir(module)
        return getattr(module, attr)

keysyms = LazyModule('keysyms', locals())

try:
    # FIXME: this is suboptimal, we load pygtk even if the user doesn't need it.
    # Maybe building the module as _gtkmodule.so isn't that bad after all.
    import _pgmgtk as gtk
    sys.modules['pgm.gtk'] = gtk
except ImportError:
    pass



# -*- mode: python; coding: utf-8 -*-
#
# Pigment Python binding webcam example
#
# Copyright © 2006, 2007, 2008 Fluendo Embedded S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# Author: Loïc Molinari <loic@fluendo.com>

import sys, pgm, gobject, gst

# Works with my cam, certainly not with yours, adapt the caps.
PIPELINE = 'v4lsrc autoprobe-fps=false ! '                            \
           'video/x-raw-yuv,width=320,height=240,framerate=225/16 ! ' \
           'ffmpegcolorspace ! '                                      \
           'pgmimagesink name="sink"'

# Terminate the mainloop on 'Escape' or 'q' key press event
def on_key_press(viewport, event, pipeline):
    if event.keyval == pgm.keysyms.q or event.keyval == pgm.keysyms.Escape:
        pgm.main_quit()

# Terminate the mainloop on a delete event
def on_delete(viewport, event, pipeline):
    pipeline.set_state(gst.STATE_NULL)
    pgm.main_quit()

def main(args):
    # OpenGL viewport creation
    gl = pgm.viewport_factory_make('opengl')
    gl.title = 'Webcam'
    gl.alpha_blending = False

    # Canvas and image drawable creation
    cvs = pgm.Canvas()
    img = pgm.Image()

    # Bind the canvas to the OpenGL viewport
    gl.set_canvas(cvs)

    # Image properties
    img.position = (0.0, 0.0, 0.0)
    img.size = (4.0, 3.0)
    img.bg_color = (0, 0, 0, 0)
    img.show()

    # Add it to the middle layer of the canvas
    cvs.add(pgm.DRAWABLE_MIDDLE, img)

    # GStreamer pipeline setup
    pipeline = gst.parse_launch(PIPELINE)
    sink = pipeline.get_by_name('sink')
    sink.set_property('image', img)
    pipeline.set_state(gst.STATE_PLAYING)

    # Let's start the mainloop
    gl.connect('delete-event', on_delete, pipeline)
    gl.connect('key-press-event', on_key_press, pipeline)
    gl.show()
    pgm.main()

if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



# -*- mode: python; coding: utf-8 -*-
#
# Pigment Python tools
#
# Copyright © 2006, 2007, 2008 Fluendo Embedded S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

# TODO: - Spline interpolation

"""
"""

from pgm.utils import classinit, maths
from pgm.timing import keyframe
import pygst
#pygst.require('0.10')
import gst

# Interpolation constants
LINEAR, SPLINE = 0, 1
ABSOLUTE, RELATIVE = 0, 1


class Modifier(object):
    """
    """

    # Allows property fget/fset/fdel/doc overriding
    __metaclass__ = classinit.ClassInitMeta
    __classinit__ = classinit.build_properties

    def __init__(self, obj=[], prop=None, timeline=[], interpolation=LINEAR,
                 operation=ABSOLUTE):
        self._lerp = None
        self._operation = operation
        self._prop_initial_values = []

        self.obj = obj
        self.prop = prop
        self.timeline = timeline
        self.interpolation = interpolation

    def __repr__(self):
        string = {LINEAR:'LINEAR', SPLINE:'SPLINE'}
        return "<modifier.Modifier(obj=%r, prop=%r, timeline=%r, " \
               "interpolation=%s)>" % (self._obj, self._prop, self._timeline,
                                       string[self._interpolation])

    # Property definitions

    def obj__get(self):
        """The list of objects"""
        return self._obj

    def obj__set(self, obj):
        if isinstance(obj, (list, tuple)):
            try:
                obj_type = type(obj[0])
                for i in obj[1:]:
                    pass
##                     if (not issubclass(obj_type, type(i))) or (not issubclass(type(i),obj_type)):
##                         error = 'Mismatching types in the list (%r vs %r)'
##                         raise TypeError, error % (type(i), obj_type)
            # obj is an empty list
            except IndexError:
                self._obj = []
            # All the objects in the list are of the same type
            else:
                self._obj = list(obj)
        else:
            gst.warning('Modifier.obj__set(): obj must be list or tuple')
            self._obj = []

    def prop__get(self):
        """The property of the object class"""
        return self._prop

    def prop__set(self, prop):
        # Check if prop is in the namespace of the obj class
        try:
            attrib = getattr(self._obj[0], prop)
            self._prop = prop
            # Store a pointer to the adapted lerp func
            if isinstance(attrib, (list, tuple)):
                self._lerp = maths.lerp_seq
            else:
                self._lerp = maths.lerp

            if self._operation == RELATIVE:
                self._save_property_initial_value()

        except TypeError:
            gst.warn('Modifier.prop__set(): prop must be string')
            self._prop = None
        except IndexError:
            gst.warn('Modifier.prop__set(): obj list is empty')
            self._prop = None
        except AttributeError:
            gst.warn('Modifier.prop__set(): No attribute %s in %r',
                     prop, self._obj[0])
            self._prop = None

    def timeline__get(self):
        """The timeline (list of keyframes) to interpolate"""
        return self._timeline

    def timeline__set(self, timeline):
        if isinstance(timeline, (list, tuple)):
            try:
                for i in timeline:
                    if not isinstance(i, keyframe.KeyFrame):
                        raise TypeError, 'Not a keyframe.KeyFrame object'
            # obj is an empty list
            except IndexError:
                self._timeline = []
            # All the objects in the list are KeyFrame
            # FIXME: A sort on the KeyFrame.time parameter is needed
            #        here because we can't assume the user give us a
            #        sorted list.
            else:
                self._timeline = timeline
        else:
            gst.warn('Modifier.timeline__set(): ' \
                     'timeline must be list or tuple')
            self._timeline = []
        if len(self._timeline) < 2:
            raise TypeError, 'timeline must at least contain 2 KeyFrame'

    def interpolation__get(self):
        """The interpolation type"""
        return self._interpolation

    def interpolation__set(self, interpolation):
        if interpolation == LINEAR or interpolation == SPLINE:
            self._interpolation = interpolation
        else:
            print 'Modifier.interpolation__set():' \
                  'Bad interpolation type, reseting to LINEAR.'
            self._interpolation = LINEAR

    def _save_property_initial_value(self):
        """
        """
        for obj in self._obj:
            value = obj.__getattribute__(self._prop)
            self._prop_initial_values.append(value)

    # Public methods

    def next_keyframe_index(self, factor):
        """
        Return the index of the next KeyFrame that will be reached in the
        timeline starting from time factor.
        """
        # We assume here that self._timeline is sorted.
        timeline_len = len(self._timeline)
        i = 0
        while self._timeline[i].time < factor:
            i += 1
            # The user gives us a partial timeline finishing before 1.0.
            if i >= timeline_len:
                return -1

        return i

    def update(self, factor):
        """
        """
        i = self.next_keyframe_index(factor)
        if i == -1:
            return

        # Expand the factor from the time range
        # [self._timeline[i-1].time, self._timeline[i].time]
        # to the range [0, 1].
        new_factor = (factor - self._timeline[i-1].time) \
                      / (self._timeline[i].time - \
                         self._timeline[i-1].time)
        # Linearly interpolate the new_factor between the KeyFrame values.
        value = self._lerp(self._timeline[i-1].value,
                           self._timeline[i].value, new_factor)

        # Let's modify the objects.
        rank = 0
        for obj in self._obj:
            if self._operation == RELATIVE:
                initial_value = self._prop_initial_values[rank]
                if isinstance(initial_value, (list, tuple)):
                    relative_value = tuple(map(lambda x,y: x+y,
                                               initial_value, value))
                else:
                    relative_value = initial_value + value
                obj.__setattr__(self._prop, relative_value)
            else:
                obj.__setattr__(self._prop, value)
            rank += 1



# -*- mode: python; coding: utf-8 -*-
#
# Pigment Python binding text example
#
# Copyright © 2006, 2007, 2008 Fluendo Embedded S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# Author: Loïc Molinari <loic@fluendo.com>

import sys, pgm, gobject

# Align the text function of the button pressed
def on_button_press(viewport, event, gl, txt):
    if event.type == pgm.BUTTON_PRESS:
        if event.button == pgm.BUTTON_LEFT:
            txt.alignment = pgm.TEXT_ALIGN_LEFT
        elif event.button == pgm.BUTTON_MIDDLE:
            txt.alignment = pgm.TEXT_ALIGN_CENTER
        elif event.button == pgm.BUTTON_RIGHT:
            txt.alignment = pgm.TEXT_ALIGN_RIGHT

# Terminate the mainloop on a delete event
def on_delete(viewport, event):
    pgm.main_quit()

def main(args):
    # OpenGL viewport creation
    gl = pgm.viewport_factory_make('opengl')
    gl.title = '60ème Festival de Cannes'

    # Canvas and text drawable creation
    cvs = pgm.Canvas()
    txt = pgm.Text(u"<b><tt>"                                                  \
                   "<span foreground=\"red\">Festival</span> "                 \
                   "<span foreground=\"green\">de</span> "                     \
                   "<span foreground=\"blue\">Cannes</span>"                   \
                   "</tt></b>\n\n"                                             \
                   u"Le jury du 60ème Festival de Cannes a décerné, dimanche " \
                   u"27 mai au soir, la Palme d'or à un film roumain, <i>4 "   \
                   u"mois, 3 semaines et 2 jours</i>, en tête d'un palmarès "  \
                   u"qui braque résolument les projecteurs sur le cinéma "     \
                   u"d'auteur le plus exigeant. Ce palmarès ignore en "        \
                   u"revanche totalement des habitués de la Croisette tels "   \
                   u"que Quentin Tarantino, Emir Kusturica, Wong Kar-wai ou "  \
                   u"les frères Coen, alors que le film de ces derniers, <i>"  \
                   u"No Country for Old Men</i>, était souvent donné favori "  \
                   u"par les rumeurs.")

    # Bind the canvas to the OpenGL viewport
    gl.set_canvas(cvs)

    # Text properties tweaking
    txt.size = (1.5, 1.5)
    txt.position = (1.25, 0.75, 0.0)
    txt.fg_color = (220, 220, 220, 255)
    txt.bg_color = (20, 20, 20, 255)

    # Change the size of the text
    txt.font_height = 1/17.0 #  1/17 to ensure around 16 lines of text.

    # A drawable needs to be shown
    txt.show()

    # Add it to the middle layer of the canvas
    cvs.add(pgm.DRAWABLE_MIDDLE, txt)

    # Let's connect our callbacks and start the mainloop
    gl.connect('button-press-event', on_button_press, gl, txt)
    gl.connect('delete-event', on_delete)
    gl.show()
    pgm.main()

if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



# -*- mode: python; coding: utf-8 -*-
#
# Pigment Python binding cairo integration example using the gradient cairo
# snippets translated to Python.
#
# Copyright © 2006, 2007, 2008 Fluendo Embedded S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# Author: Loïc Molinari <loic@fluendo.com>

import sys, math, array, pgm, cairo

# Terminate the mainloop on a delete event
def on_delete(viewport, event):
    pgm.main_quit()

# Draw a gradient in a memory buffer with Cairo
def cairo_drawing(width, height):
    # Store the Cairo surface in an array
    data = array.array('c', chr(0) * width * height * 4)
    surface = cairo.ImageSurface.create_for_data (data, cairo.FORMAT_ARGB32,
                                                  width, height, width * 4)
    # Create the context
    ctx = cairo.Context(surface)
    ctx.scale(width, height)

    # Draw a sphere
    pat = cairo.RadialGradient(0.45, 0.4, 0.1, 0.4, 0.4, 0.5)
    pat.add_color_stop_rgba(0, 0.8, 0.8, 1, 1)
    pat.add_color_stop_rgba(1, 0, 0, 1, 1)
    ctx.set_source(pat)
    ctx.arc(0.5, 0.5, 0.3, 0, 2 * math.pi)
    ctx.fill()

    # And return the buffer as a Python string
    return data.tostring()

# Entry point
def main():
    # OpenGL viewport creation
    gl = pgm.viewport_factory_make('opengl')
    gl.title = 'Sphere'

    # Canvas and image drawable creation
    cvs = pgm.Canvas()
    cvs.size = (800.0, 600.0)

    # Bind the canvas to the OpenGL viewport
    gl.set_canvas(cvs)

    # Draw a gradient with cairo and put it in a Pigment Image
    img = pgm.Image()
    data = cairo_drawing(400, 400)
    img.set_from_buffer(pgm.IMAGE_BGRA, 400, 400, 400 * 4, 400 * 400 * 4, data)
    img.bg_color = (255, 255, 255, 0)
    img.position = (200.0, 100.0, 0.0)
    img.size = (400.0, 400.0)
    img.show()

    # Add the drawable to the middle layer of the canvas
    cvs.add(pgm.DRAWABLE_MIDDLE, img)

    # Let's connect our callbacks and start the mainloop
    gl.connect('delete-event', on_delete)
    gl.show()
    pgm.main()

if __name__ == '__main__':
    sys.exit(main())



# -*- mode: python; coding: utf-8 -*-
#
# Pigment Python binding timing example
#
# Copyright © 2006, 2007, 2008 Fluendo Embedded S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# Author: Loïc Molinari <loic@fluendo.com>

import sys, pgm, gobject
from pgm.timing import controller, modifier, keyframe

fullscreen = False

# Terminate the mainloop on a delete event
def on_delete(viewport, event, ctrl):
    ctrl.stop()
    pgm.main_quit()

# Key press handler
def on_key_press(viewport, event):
    global fullscreen
    if event.keyval == pgm.keysyms.f:
        fullscreen = not fullscreen
        viewport.fullscreen = fullscreen

def main(args):
    # OpenGL viewport creation
    gl = pgm.viewport_factory_make('opengl')
    gl.title = 'Animated image'

    # Canvas and image drawable creation
    cvs = pgm.Canvas()
    img = pgm.Image()

    # Bind the canvas to the OpenGL viewport
    gl.set_canvas(cvs)

    # Load an image
    img.set_from_file('examples/pictures/fluendo.png')
    img.bg_color = (255, 255, 255, 0)

    # Center the image
    img.position = (1.5, 1.0, 0.0)

    # A drawable needs to be shown
    img.show()

    # Add it to the middle layer of the canvas
    cvs.add(pgm.DRAWABLE_MIDDLE, img)

    # Create a animation position timeline
    tml = [keyframe.KeyFrame(2.0, 0.0),
           keyframe.KeyFrame(0.25, 1.0)]
    mdf = modifier.Modifier([img], 'y', tml)

    # Create the animation controller
    ctrl = controller.Controller(duration=1000, resolution=5,
                                 repeat_behavior=controller.REVERSE,
                                 repeat_count=controller.INFINITE,
                                 transformation=controller.DECELERATE,
                                 modifiers=[mdf])

    # Start the controller
    ctrl.start()

    # Let's start the mainloop
    gl.connect('delete-event', on_delete, ctrl)
    gl.connect('key-press-event', on_key_press)
    gl.show()
    pgm.main()

if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



# -*- mode: python; coding: utf-8 -*-
#
# Pigment Python tools
#
# Copyright © 2006, 2007, 2008 Fluendo Embedded S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

from pgm.utils import classinit
import pgm
import math
import gobject

class Group(gobject.GObject):

    __metaclass__ = classinit.GClassInitMeta
    __classinit__ = classinit.build_properties
    
    __gsignals__ = {
        'drag-begin': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_BOOLEAN,
                (gobject.TYPE_FLOAT, gobject.TYPE_FLOAT, gobject.TYPE_FLOAT,
                pgm.ButtonType, gobject.TYPE_UINT)),
        'drag-motion': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_BOOLEAN,
                (gobject.TYPE_FLOAT, gobject.TYPE_FLOAT, gobject.TYPE_FLOAT,
                pgm.ButtonType, gobject.TYPE_UINT)),
        'drag-end': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_BOOLEAN,
                (gobject.TYPE_FLOAT, gobject.TYPE_FLOAT, gobject.TYPE_FLOAT,
                pgm.ButtonType, gobject.TYPE_UINT)),
        'clicked': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_BOOLEAN,
                (gobject.TYPE_FLOAT, gobject.TYPE_FLOAT, gobject.TYPE_FLOAT,
                pgm.ButtonType, gobject.TYPE_UINT)),
        'pressed': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_BOOLEAN,
                (gobject.TYPE_FLOAT, gobject.TYPE_FLOAT, gobject.TYPE_FLOAT,
                pgm.ButtonType, gobject.TYPE_UINT)),
        'released': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_BOOLEAN,
                (gobject.TYPE_FLOAT, gobject.TYPE_FLOAT, gobject.TYPE_FLOAT,
                pgm.ButtonType, gobject.TYPE_UINT)),
        'double-clicked': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_BOOLEAN,
                (gobject.TYPE_FLOAT, gobject.TYPE_FLOAT, gobject.TYPE_FLOAT,
                pgm.ButtonType, gobject.TYPE_UINT)),
    }

    def __init__(self, canvas=None, layer=pgm.DRAWABLE_MIDDLE):
        super(Group, self).__init__()
        self._children = []
        self._position = (0.0, 0.0, 0.0)
        self._opacity = 255
        self._screen_visible = False
        self._real_visible = False
        self._parent = None
        self._canvas = canvas
        self._layer = layer

    def regenerate(self):
        for child in self._children:
            child.regenerate()

    def children__get(self):
        return self._children


    def position__get(self):
        return self._position


    def position__set(self, value):
        self._position = value
        for drawable in self._children:
            if not isinstance(drawable, Group):
                drawable.position_offset = self.absolute_position
            drawable.position = drawable.position


    def x__get(self):
        return self._position[0]


    def x__set(self, value):
        new_position = (value, self._position[1], self._position[2])
        self.position = new_position


    def y__get(self):
        return self._position[1]


    def y__set(self, value):
        new_position = (self._position[0], value, self._position[2])
        self.position = new_position


    def z__get(self):
        return self._position[2]


    def z__set(self, value):
        new_position = (self._position[0], self._position[1], value)
        self.position = new_position


    def absolute_position__get(self):
        if self._parent == None:
            return self._position
        else:
            absolute_position = tuple(map(lambda x,y: x+y,
                                               self._position,
                                               self._parent.absolute_position))
            return absolute_position

    def opacity__get(self):
            return self._opacity

    def opacity__set(self, value):
        factor = float(value) / 255
        self._opacity = value
        for drawable in self._children:
            if isinstance (drawable, Group):
                drawable.opacity = drawable.opacity
            else:
                drawable.opacity_factor_offset__set(self.absolute_opacity_factor__get())

    def absolute_opacity_factor__get(self):
        if self._parent == None:
            return self._opacity / 255.0
        else:
            opacity_factor = self._opacity * self._parent.absolute_opacity_factor__get() / 255.0
            return opacity_factor


    def _set_visibility_from_parent(self, visible, from_parent):

        if not from_parent:
            self._real_visible = visible

        if self._parent == None:
            self._real_visible = visible
            self._screen_visible = visible
        elif self._parent.visible == True and self._real_visible == True:
            self._screen_visible = True
        else:
            self._screen_visible = False


        for drawable in self._children:
            if isinstance(drawable, Group):
                drawable._set_visibility_from_parent(self._screen_visible, True)
            else:
                drawable.parent_visibility = self._screen_visible


    def visible__get(self):
        return self._screen_visible


    def visible__set(self, value):
        self._set_visibility_from_parent(value, False)

    def size__set(self, size):
        current_size = self.size

        if current_size[0] == 0.0 or current_size[1] == 0.0:
            # FIXME: do something more sensible
            return

        kx, ky = (size[0]/current_size[0], size[1]/current_size[1])

        for child in self._children:
            child.x *= kx
            child.y *= ky
            w, h = child.size
            child.size = (w*kx, h*ky)

    def width__set(self, width):
        height = self.size[1]
        self.size = (width, height)

    def height__set(self, height):
        width = self.size[0]
        self.size = (width, height)

    def size__get(self):
        if len(self._children) == 0:
            return (0.0, 0.0)

        child = self._children[0]
        min_x = child.x
        min_y = child.y
        max_x = child.x + child.size[0]
        max_y = child.y + child.size[1]
        for child in self._children:
             min_x = min(min_x, child.x)
             min_y = min(min_y, child.y)
             max_x = max(max_x, child.x + child.size[0])
             max_y = max(max_y, child.y + child.size[1])

        return (max_x-min_x, max_y-min_y)

    def width__get(self):
        return self.size[0]

    def height__get(self):
        return self.size[1]

    def parent__get(self):
        return self._parent

    def add(self, drawable, forward_signals=False):
        if drawable not in self._children:
            self._children.append(drawable)
            if isinstance(drawable, Group):
                drawable._parent = self
                drawable.position = drawable.position
                drawable.opacity = drawable.opacity
                drawable.canvas = self._canvas
            else:
                if self._canvas != None:
                    self._canvas.add(self._layer, drawable)
                drawable.position_offset = self.absolute_position
                drawable.parent_visibility = self.visible
                drawable.opacity_factor_offset__set(self.absolute_opacity_factor__get())

            if forward_signals:
                for signal_name in ('clicked', 'pressed', 'released',
                                    'drag-begin', 'drag-motion', 'drag-end'):
                    drawable.connect(signal_name,
                            self._proxy_child_signal, signal_name) 

    def remove(self, drawable):
        if drawable in self._children:
            self._children.remove(drawable)
            if isinstance(drawable, Group):
                drawable._parent = None
                drawable.canvas = None
            else:
                if self._canvas != None:
                    self._canvas.remove(drawable)

    def empty(self):
        for child in self._children:
            if isinstance(child, Group):
                child.empty()
            else:
                self._canvas.remove(child)

        self._children = []

    def canvas__set(self, canvas):
        for child in self._children:
            if isinstance(child, Group):
                child.canvas = canvas
            else:
                if canvas != None:
                    canvas.add(self._layer, child)
                elif self._canvas != None:
                    self._canvas.remove(child)

        self._canvas = canvas

    def canvas__get(self):
        return self._canvas

    def _proxy_child_signal(self, drawable, x, y, z, button, time, signal_name):
        return self.emit(signal_name, x, y, z, button, time)



# -*- mode: python; coding: utf-8 -*-
#
# Pigment Python tools
#
# Copyright © 2006, 2007, 2008 Fluendo Embedded S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

# TODO: - Timing (ac/de)celeration

"""
"""

from pgm.utils import classinit, maths
from pgm.timing import modifier
import math, time, gobject
import pygst
#pygst.require('0.10')
import gst


# Repeat behavior constants
FORWARD, REVERSE = 0, 1
# End behavior constants
BEGIN, HOLD, END = 2, 3, 4
# Transformation constants
LINEAR, ACCELERATE, DECELERATE, SMOOTH = 5, 6, 7, 8
# Repeat constant
INFINITE = -1


class Controller(object):
    """
    """

    # Allows property fget/fset/fdel/doc overriding
    __metaclass__ = classinit.ClassInitMeta
    __classinit__ = classinit.build_properties

    def __init__(self, begin=0, duration=1000, resolution=5,
                 repeat_behavior=FORWARD, repeat_count=1, end_behavior=BEGIN,
                 modifiers=[], transformation=LINEAR, end_callback=None):
        self.begin = begin
        self.duration = duration
        self.resolution = resolution
        self.repeat_behavior = repeat_behavior
        self.repeat_count = repeat_count
        self.end_behavior = end_behavior
        self.modifiers = modifiers
        self.transformation = transformation
        self.end_callback = end_callback
        self._start_time = 0
        self._update_time = 0
        self._begin_time = 0
        self._repeat_counter = 0
        self._started = False
        self._tag = None

    def __repr__(self):
        string = {FORWARD:'FORWARD', REVERSE:'REVERSE',
                  BEGIN:'BEGIN', HOLD:'HOLD', END:'END'}
        return "<controller.Controller(begin=%r, duration=%r, resolution=%r, " \
               "repeat_behavior=%s, repeat_count=%r, end_behavior=%s, "        \
               "modifiers=%r)>"                                                \
               % (self._begin, self._duration, self._resolution,
                  string[self._repeat_behavior], self._repeat_count,
                  string[self._end_behavior], self._modifiers)

    # Property definitions

    def begin__get(self):
        """The time (in ms) to wait after a start()"""
        return self._begin

    def begin__set(self, begin):
        self._begin = max(0, int(begin))

    def duration__get(self):
        """The duration (in ms) of each animation"""
        return self._duration

    def duration__set(self, duration):
        self._duration = max(0, int(duration))

    def resolution__get(self):
        """The update frequency (in ms)"""
        return self._resolution

    def resolution__set(self, resolution):
        self._resolution = max(0, int(resolution))

    def repeat_behavior__get(self):
        """The repeat behavior at the end of an animation"""
        return self._repeat_behavior

    def repeat_behavior__set(self, repeat_behavior):
        if repeat_behavior == FORWARD or repeat_behavior == REVERSE:
            self._repeat_behavior = repeat_behavior
        else:
            self._repeat_behavior = FORWARD

    def repeat_count__get(self):
        """The number of iteration"""
        return self._repeat_count

    def repeat_count__set(self, repeat_count):
        # -1 is the INFINITE constant for the repeat mode
        self._repeat_count = max(-1, int(repeat_count))

    def end_behavior__get(self):
        """The behavior at the end of an animation"""
        return self._end_behavior

    def end_behavior__set(self, end_behavior):
        if end_behavior == BEGIN or end_behavior == HOLD \
               or end_behavior == END:
            self._end_behavior = end_behavior

    def modifiers__get(self):
        """The list of object modifiers"""
        return self._modifiers

    def modifiers__set(self, modifiers):
        if isinstance(modifiers, (list, tuple)):
            try:
                for i in modifiers:
                    if not isinstance(i, modifier.Modifier):
                        raise TypeError, 'Not a modifier.Modifier object'
            # modifiers is an empty list
            except IndexError:
                self._modifiers = []
            # All the objects in the list are Modifier
            else:
                self._modifiers = list(modifiers)
        else:
            # FIXME: I would avoid using gst.warn
            gst.warn('Controller.set_modifiers(): modifiers must be list or ' \
                     'tuple')
            self._modifiers = []

    def transformation__get(self):
        """The transformation type"""
        return self._transformation

    def transformation__set(self, transformation):
        if transformation in (LINEAR, ACCELERATE, DECELERATE, SMOOTH):
            self._transformation = transformation

    def end_callback__get(self):
        """
        The callback called at the end of the controller
        (duration*repeat_count ms).
        """
        return self._end_callback

    def end_callback__set(self, end_callback):
        if callable(end_callback):
            self._end_callback = end_callback
        else:
            self._end_callback = None

    def started__get(self):
        """Is the controller started ?"""
        return self._started

    # Public methods

    def start(self):
        """
        """
        self._start_time = long(time.time() * 1000)
        self._update_time = self._start_time
        self._begin_time = self._start_time + self._begin
        self._repeat_counter = 0
        self._started = True
        # Add the source to the mainloop
        self._tag = gobject.timeout_add(self._resolution, self._update)

    def stop(self):
        """
        """
        if self._started and self._tag != None:
            # Remove the source from the mainloop
            gobject.source_remove(self._tag)
            self._tag = None

            # Respect the end behavior by sending the appropriate factor to
            # the modifiers.
            if self._end_behavior == BEGIN:
                factor = 0.0
            elif self._end_behavior == END:
                factor = 1.0
            elif self._end_behavior == HOLD:
                if self._duration != 0:
                    factor = self._compute_factor()
                else:
                    factor = 1.0
            for modifier in self._modifiers:
                modifier.update(factor)

            # Reset internal data
            self._start_time = 0
            self._update_time = 0
            self._begin_time = 0
            self._repeat_counter = 0
            self._started = False

            # Let's call the end callback
            if self._end_callback:
                self._end_callback(self)

    # Private methods

    def _update(self):
        # Compute elapsed time
        self._update_time = long(time.time() * 1000)
        diff_begin_time = self._update_time - self._begin_time

        # Is the controller duration elapsed ?
        if diff_begin_time >= self._duration:
            self._repeat_counter += 1
            # Repeat the animation
            if (self._repeat_count == INFINITE \
               or self._repeat_counter < self._repeat_count) \
               and self._duration != 0:
                self._begin_time += self._duration
            # Stop the animation
            else:
                self.stop()
                return False

        factor = self._compute_factor()

        # Update the modifiers
        for modifier in self._modifiers:
            modifier.update(factor)

        # Return True to keep the source in the mainloop
        return True

    def _compute_factor(self):
        # Compute the factor
        elapsed_time = self._update_time - self._begin_time
        factor = min(1.0, float(elapsed_time) / self._duration)
        # Reverse the factor if we are in REVERSE mode
        if self._repeat_behavior == REVERSE and self._repeat_counter % 2:
            factor = 1.0 - factor
        # Apply the transformation on the time
        if self._transformation == SMOOTH:
            sinus = math.sin(maths.lerp(-maths.PI2, maths.PI2, factor))
            factor = (sinus + 1.0) * 0.5
        elif self._transformation == DECELERATE:
            factor = math.sin(maths.lerp(0, maths.PI2, factor))
        elif self._transformation == ACCELERATE:
            factor = math.sin(maths.lerp(-maths.PI2, 0, factor)) + 1.0

        return factor



# -*- mode: python; coding: utf-8 -*-
#
# Pigment Python tools
#
# Copyright © 2006, 2007, 2008 Fluendo Embedded S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

from pgm.graph.group import Group
import gobject

class Button(Group):
    """
    A Button is a Group enhanced with focusing possibilities.

    # TODO: this does not belong to Button but more into something, we would
    # call a WIDGET

    @ivar focus: is this Button focused?
    @type focus: bool
    """

    __gsignals__ = {'focus': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_BOOLEAN,
                    (gobject.TYPE_BOOLEAN,))}

    def __init__(self):
        self._focus = False
        self._focus_on_clicked = True
        super(Button, self).__init__()

    def clicked(self, drawable, x, y, z, button, time):
        if self._focus_on_clicked:
            self.focus = True
        self.emit('clicked', x, y, z, button, time)
        return True

    def pressed(self):
        pass

    def focus__get(self):
        return self._focus

    def focus__set(self, value):

        if self._focus == value:
            return

        self._focus = value
        self.emit('focus', value)




# -*- mode: python; coding: utf-8 -*-
#
# Pigment Python tools
#
# Copyright © 2006, 2007, 2008 Fluendo Embedded S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

import pgm

from pgm.graph.group import Group
from pgm.graph.image import Image
from pgm.utils import image as image_utils
from pgm.widgets import const

class SlicedImage(Group):

    def __init__(self, orientation=const.VERTICAL, top_file=None,
                 bottom_file=None, body_file=None):
        self._size = (0,0)
        Group.__init__(self)

        # invisble image used to handle mouse related signals, mainly.
        self._invisible = Image()
        self.add(self._invisible)
        self._invisible.size = self.size

        self._body = Image()
        self.add(self._body)
        self._body.bg_color = (255, 0, 0, 0)
        self._body.layout = pgm.IMAGE_FILLED
        self._body.interp = pgm.IMAGE_NEAREST
        self._body.connect("pixbuf-loaded", self._body_loaded)
        self._body_loaded = False

        self._bottom = Image()
        self.add(self._bottom)
        self._bottom.layout = pgm.IMAGE_SCALED
        self._bottom.layout = pgm.IMAGE_FILLED
        self._bottom.bg_color = (0, 255, 0, 0)
        self._bottom.interp = pgm.IMAGE_NEAREST
        self._bottom.connect("pixbuf-loaded", self._bottom_loaded)
        self._bottom_loaded = False

        self._top = Image()
        self.add(self._top)
        self._top.layout = pgm.IMAGE_SCALED
        self._top.layout = pgm.IMAGE_FILLED
        self._top.bg_color = (0, 0, 255, 0)
        self._top.interp = pgm.IMAGE_NEAREST
        self._top.connect("pixbuf-loaded", self._top_loaded)
        self._top_loaded = False

        self._orientation = orientation
        self.top_file = top_file
        self.bottom_file = bottom_file
        self.body_file = body_file
        
    def connect(self, signal_name, callback, *args, **kw):
        # Forward signal connections to self._invisible
        return self._invisible.connect(signal_name, callback, *args, **kw)

    def orientation__set(self, orientation):
        self._orientation = orientation
        self._compute_layout()

    def orientation__get(self):
        return self._orientation

    def size__set(self, size):
        self._size = size
        self._compute_layout()

    def size__get(self):
        return self._size

    def width__set(self, width):
        self._size = (width, self._size[1])
        self._compute_layout()

    def width__get(self):
        return self._size[0]

    def height__set(self, height):
        #Group.height__set(self, height)
        self._size = (self._size[0], height)
        self._compute_layout()
        
    def width__get(self):
        return self._size[0]

    def _compute_layout(self):
        if self._top_loaded and self._body_loaded and self._bottom_loaded:
            self._top_layout()
            self._body_layout()
            self._bottom_layout()
            self._top.visible = True
            self._body.visible = True
            self._bottom.visible = True

    def _bottom_loaded(self, img):
        self._bottom_loaded = True
        self._compute_layout()

    def _top_loaded(self, img):
        self._top_loaded = True
        self._compute_layout()

    def _body_loaded(self, img):
        self._body_loaded = True
        self._compute_layout()

    def _bottom_layout(self):
        if self._orientation == const.VERTICAL:
            self._bottom.alignment = pgm.IMAGE_BOTTOM_CENTER
            self._bottom.width = self.width

            y = self._body.height + self._body.y
            self._bottom.position = (0, y, 0)
            self._bottom.height = image_utils.get_height_from_par(self._bottom)
        else:
            self._bottom.alignment = pgm.IMAGE_RIGHT
            self._bottom.height = self.height
            x = self._body.width + self._body.x
            self._bottom.position = (x, 0, 0)
            self._bottom.width = image_utils.get_width_from_par(self._bottom)

    def _top_layout(self):
        self._top.position = (0, 0, 0)

        if self._orientation == const.VERTICAL:
            self._top.alignment = pgm.IMAGE_TOP_CENTER
            self._top.width = self.width

            self._top.height = image_utils.get_height_from_par(self._top)
        else:
            self._top.alignment = pgm.IMAGE_LEFT
            self._top.height = self.height

            self._top.width = image_utils.get_width_from_par(self._top)

    def _body_layout(self):
        if self._orientation == const.VERTICAL:
            self._body.width = self.width

            # assume that self._bottom.height == self._top.height
            self._body.height = self.height - self._top.height*2.0# - self._bottom.height
            self._body.position = (0, self._top.height, 0)
        else:
            self._body.height = self.height

            # assume that self._bottom.width == self._top.width
            self._body.width = self.width - self._top.width*2.0# - self._bottom.width
            self._body.position = (self._top.width, 0, 0)

    def top_file__set(self, top_file):
        if top_file is not None:
            self._top.set_from_file(top_file)

    def bottom_file__set(self, bottom_file):
        if bottom_file is not None:
            self._bottom.set_from_file(bottom_file)

    def body_file__set(self, body_file):
        if body_file is not None:
            self._body.set_from_file(body_file)


if __name__ == "__main__":
    import os, os.path

    def on_key_press(viewport, event):
        if event.type == pgm.KEY_PRESS:
            # quit on q or ESC
            if event.keyval == pgm.keysyms.q or \
               event.keyval == pgm.keysyms.Escape:
                pgm.main_quit()
            
            elif event.keyval == pgm.keysyms.f:
                viewport.fullscreen = not viewport.fullscreen

            elif event.keyval == pgm.keysyms.h:
                widget.width += 0.1

            elif event.keyval == pgm.keysyms.n:
                widget.width -= 0.1

            elif event.keyval == pgm.keysyms.g:
                widget.height += 0.1

            elif event.keyval == pgm.keysyms.b:
                widget.height -= 0.1

    def on_delete(viewport, event):
        pgm.main_quit()

    # OpenGL viewport creation
    factory = pgm.ViewportFactory('opengl')
    gl = factory.create()
    gl.title = 'Sliced image widget'

    # Canvas and image drawable creation
    canvas = pgm.Canvas()

    # Bind the canvas to the OpenGL viewport
    gl.set_canvas(canvas)
    gl.show()

    dir = "/home/ben/dev/elisa/elisa/plugins/bad/raval_frontend"
    top = "theme/selector-left.png"
    bottom = "theme/selector-right.png"
    body = "theme/selector-body.png"

    widget = SlicedImage(orientation=const.HORIZONTAL)
    widget.top_file = os.path.join(dir, top)
    widget.bottom_file = os.path.join(dir, bottom)
    widget.body_file = os.path.join(dir, body)
    widget.width = canvas.width/2.0
    widget.height = canvas.height/2.0
    widget.x = (canvas.width - widget.width)/2.0
    widget.y = (canvas.height - widget.height)/2.0
    widget.visible = True
    widget.canvas = canvas

    # Let's start a mainloop
    gl.connect('key-press-event',
               on_key_press)
    gl.connect('delete-event', on_delete)
    pgm.main()



# -*- mode: python; coding: utf-8 -*-
#
# Pigment Python tools
#
# Copyright © 2006, 2007, 2008 Fluendo Embedded S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

import pgm
from pgm.graph.text import Text

class Console(Text):

    def __init__(self):
        Text.__init__(self)

        self._lines = [""]
        self._current_line = 0
        self._visible_lines = 5

        # default style
        self.wrap = pgm.TEXT_WRAP_CHAR
        self.font_height = self.height / self._visible_lines * 0.7
        self.size = (2.0, 1.0)
        self.fg_color = (200, 200, 200, 255)
        self.bg_color = (50, 50, 50, 255)

    def append_to_current_line(self, text):
        self._lines[self._current_line] += text
        self._refresh()

    def delete_previous_character(self):
        pass

    def next_line(self):
        self._current_line += 1
        self._lines.append("")
        self._refresh()

    def clear(self):
        pass

    def clear_current_line(self):
        pass

    def set_visible_lines(self, lines):
        self.font_height = self.height / lines
        self._visible_lines = lines
        self._refresh()

    def _refresh(self):
        buffer = ""
        for line in self._lines[-self._visible_lines:]:
            buffer += line
            buffer += "\n"
        self.label = buffer

if __name__ == "__main__":
    import pgm
    import gobject

    def on_key_press(viewport, event, gl, widget):
        if event.type == pgm.KEY_PRESS:
            if event.keyval == pgm.keysyms.Escape:
                pgm.main_quit()

            elif event.keyval == pgm.keysyms.Return:
                widget.next_line()

            else:
                widget.append_to_current_line(unichr(event.keyval))


    def on_delete(viewport, event):
        pgm.main_quit()


    # OpenGL viewport creation
    factory = pgm.ViewportFactory('opengl')
    gl = factory.create()
    gl.title = 'Console widget test'

    # Canvas and image drawable creation
    canvas = pgm.Canvas()

    # Bind the canvas to the OpenGL viewport
    gl.set_canvas(canvas)
    gl.show()

    console_widget = Console()
    console_widget.position = (0.2, 0.5, 0.0)
    console_widget.visible = True

    canvas.add(pgm.DRAWABLE_MIDDLE, console_widget)

    # Let's start a mainloop
    gl.connect('key-press-event',
               on_key_press,
               gl,
               console_widget)
    gl.connect('delete-event', on_delete)
    pgm.main()



# -*- mode: python; coding: utf-8 -*-
#
# Pigment Python binding mapping matrix example
#
# Copyright © 2006, 2007, 2008 Fluendo Embedded S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# Author: Loïc Molinari <loic@fluendo.com>

import sys, math, pgm, gobject

# Transform the mapping matrix of the image
def transform_mapping_matrix_cb(image, transformation):
    image.mapping_matrix *= transformation
    return True

# Start animating the mapping matrix when the pixbuf is loaded
def on_pixbuf_loaded(image, transformation):
    gobject.timeout_add(10, transform_mapping_matrix_cb, image, transformation)

# Terminate the mainloop on a delete event
def on_delete(viewport, event):
    pgm.main_quit()

# Entry point
def main(args):
    # OpenGL viewport creation
    gl = pgm.viewport_factory_make('opengl')
    gl.connect('delete-event', on_delete)
    gl.size = (600, 600)
    gl.title = 'Mapping matrix'

    # Canvas creation
    cvs = pgm.Canvas()
    cvs.size = (600, 600)
    gl.set_canvas(cvs)

    # Transformation matrix creation
    t = pgm.mat4x4_new_identity()
    t.translate(0.5, 0.5, 0.0)
    t.rotate_z(math.pi / 180.0)
    t.translate(-0.5, -0.5, 0.0)

    # Image creation
    img = pgm.Image()
    img.set_from_file('examples/pictures/meiko.jpg')
    img.connect('pixbuf-loaded', on_pixbuf_loaded, t)
    img.size = (400, 400)
    img.position = (100, 100, 0)
    img.bg_color = (255, 255, 255, 0)
    cvs.add(pgm.DRAWABLE_MIDDLE, img)

    # Show the objects and start the main loop
    img.show()
    gl.show()
    pgm.main()

if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



# -*- mode: python; coding: utf-8 -*-
#
# Pigment Python binding video example
#
# Copyright © 2006, 2007, 2008 Fluendo Embedded S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# Author: Loïc Molinari <loic@fluendo.com>

import sys, pgm, gobject, gst

fps_label = None

# Key press handler
def on_key_press(viewport, event, pipeline):
    # Terminate the mainloop on 'Escape' or 'q' key press event
    if event.keyval == pgm.keysyms.q or event.keyval == pgm.keysyms.Escape:
        pgm.main_quit()
    # Switch fullscreen state
    if event.keyval == pgm.keysyms.f:
        viewport.fullscreen = not viewport.fullscreen
    # Toggle fps display
    elif event.keyval == pgm.keysyms.p:
        global fps_label
        canvas = viewport.get_canvas()
        if fps_label == None:
            add_fps_display(canvas, viewport)
        else:
            canvas.remove(fps_label)
            fps_label = None

# Drag motion handler
def on_drag_motion(viewport, event, pipeline):
    # Accept the drag if the extension of the first uri is a media file
    if event.uri[0][-3:] in ("avi", "mpg", "ogg", "mkv", "mov", "wmv", "flv"):
        return True

# Drag drop handler
def on_drag_drop(viewport, event, pipeline):
    pipeline.set_state(gst.STATE_READY)
    pipeline.set_property('uri', event.uri[0])
    pipeline.set_state(gst.STATE_PLAYING)

# Terminate the mainloop on a delete event
def on_delete(viewport, event, pipeline):
    pipeline.set_state(gst.STATE_NULL)
    pgm.main_quit()

def add_fps_display(canvas, viewport):
    global fps_label
    fps_label = pgm.Text()
    fps_label.bg_a = 0
    fps_label.width = canvas.width/10.0
    fps_label.height = canvas.height/20.0
    fps_label.x = canvas.width - fps_label.width
    fps_label.y = canvas.height - fps_label.height
    fps_label.visible = True
    canvas.add(pgm.DRAWABLE_NEAR, fps_label)
    refresh_fps_display(viewport)
    gobject.timeout_add(1000, refresh_fps_display, viewport)

def refresh_fps_display(viewport):
    global fps_label
    if fps_label != None:
        fps_label.label = "%d fps" % viewport.frame_rate
        return True
    else:
        return False

def main(args):
    # OpenGL viewport creation
    gl = pgm.viewport_factory_make('opengl')
    gl.title = "Video drag 'n drop"
    gl.alpha_blending = False
    screen_ratio = float(gl.screen_size_mm[1])/gl.screen_size_mm[0]
    gl.size = (800, 800*screen_ratio)

    # Canvas and image drawable creation
    cvs = pgm.Canvas()
    cvs.size = (1.0, 1.0*screen_ratio)
    img = pgm.Image()

    # Bind the canvas to the OpenGL viewport
    gl.set_canvas(cvs)

    # Image properties
    img.position = (0.0, 0.0, 0.0)
    img.size = cvs.size
    img.bg_color = (0, 0, 0, 0)

    # A drawable needs to be shown
    img.show()

    # Add it to the middle layer of the canvas
    cvs.add(pgm.DRAWABLE_MIDDLE, img)

    # GStreamer pipeline setup
    sink = gst.element_factory_make('pgmimagesink')
    pipeline = gst.element_factory_make('playbin')
    pipeline.set_property('video-sink', sink)
    sink.set_property('image', img)
    if (len(sys.argv) > 1):
        pipeline.set_property('uri', sys.argv[1])
        pipeline.set_state(gst.STATE_PLAYING)

    # Let's start the mainloop
    gl.connect('delete-event', on_delete, pipeline)
    gl.connect('key-press-event', on_key_press, pipeline)
    gl.connect('drag-motion-event', on_drag_motion, pipeline)
    gl.connect('drag-drop-event', on_drag_drop, pipeline)
    gl.show()
    pgm.main()

if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



# -*- mode: python; coding: utf-8 -*-
#
# Pigment Python binding multiple viewports example
#
# Copyright © 2006, 2007, 2008 Fluendo Embedded S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# Author: Loïc Molinari <loic@fluendo.com>

import sys, pgm

# Terminate the mainloop on a delete event
def on_delete(viewport, event):
    pgm.main_quit()

# Display the event and the viewport on a motion-notify event
def on_motion_notify(viewport, event):
    print 'Received', event, 'on', viewport

def main(args):
    factory = pgm.ViewportFactory('opengl')

    # OpenGL viewport 1 creation
    gl1 = factory.create()
    gl1.title = 'OpenGL viewport <1>'
    gl1.connect('delete-event', on_delete)
    gl1.connect('motion-notify-event', on_motion_notify)

    # OpenGL viewport 2 creation
    gl2 = factory.create()
    gl2.title = 'OpenGL viewport <2>'
    gl2.connect('delete-event', on_delete)
    gl2.connect('motion-notify-event', on_motion_notify)

    # Canvas and image drawable creation
    cvs = pgm.Canvas()
    img = pgm.Image()

    # Load an image from a file descriptor
    img.set_from_file('examples/pictures/fluendo.png')

    # Center the image
    img.position = (1.5, 1.0, 0.0)

    # A drawable needs to be shown
    img.show()

    # Add it to the middle layer of the canvas
    cvs.add(pgm.DRAWABLE_MIDDLE, img)

    # Bind the canvas to the viewports
    gl1.set_canvas(cvs)
    gl2.set_canvas(cvs)

    # Let's start the mainloop
    gl1.show()
    gl2.show()
    pgm.main()

if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



# -*- mode: python; coding: utf-8 -*-
#
# Pigment Python tools
#
# Copyright © 2006, 2007, 2008 Fluendo Embedded S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

import pgm
from pgm.graph.group import Group
from pgm.graph.image import Image
from pgm.timing import implicit

import math

from pgm.widgets.list_ng import List

class CircularList(List):

    def compute_width(self, index):
        return self._width / self._visible_range_size

    def compute_height(self, index):
        return self._height / self._visible_range_size

    def compute_x(self, index):
        offset = math.pi/2.0
        a = index/self._visible_range_size*math.pi*2.0 + offset
        x = (1.0+math.cos(a))/2.0*(self._width-self._widget_width)
        return x

    def compute_y(self, index):
        offset = math.pi/2.0
        a = index/self._visible_range_size*math.pi*2.0 + offset
        y = (1.0+math.sin(a))/2.0*(self._height-self._widget_height)

        epsilon = 1.0
        max_y = 1.0
        middle = self._visible_range_size/2.0

        if index > middle-epsilon and index < middle+epsilon:
            if index > middle:
                y += 1.0-(index-middle)/epsilon*max_y
            elif index <= middle:
                y += 1.0+(index-middle)/epsilon*max_y

        return y

    def compute_z(self, index):
        epsilon = 0.5
        middle = self._visible_range_size/2.0
        if index > middle-epsilon and index < middle+epsilon:
            return 0.1
        else:
            return 0.0

    def compute_opacity(self, index):
        if index < -1 or index > self._visible_range_size:
            return 0

        middle = self._visible_range_size/2.0
        max_opacity = 255
        step = max_opacity / middle

        if index > middle:
            index = -index + 2.0*middle

        opacity = index * step

        opacity = min(opacity, 255)
        opacity = max(opacity, 0)
        return opacity

    def compute_zoom(self, index):
        epsilon = 1.0
        max_zoom = 4.0
        min_zoom = 1.0
        middle = self._visible_range_size/2.0

        if index > middle-epsilon and index < middle+epsilon:
            if index > middle:
                zoom = (epsilon-(index-middle))/epsilon*(max_zoom-min_zoom) + min_zoom
            elif index <= middle:
                zoom = (epsilon+(index-middle))/epsilon*(max_zoom-min_zoom) + min_zoom
        else:
            zoom = 1.0

        zoom = max(zoom, 1.0)
        return zoom

    def layout_widget(self, widget, position):
        zoom = self.compute_zoom(position)
        width = self._widget_width*zoom
        height = self._widget_height*zoom
        x = self.compute_x(position)-(width-self._widget_width)/2.0
        y = self.compute_y(position)-(height-self._widget_height)/2.0
        z = self.compute_z(position)
        opacity = self.compute_opacity(position)

        # update widget properties
        widget.position = (x, y, z)
        widget.size = (width, height)
        widget.opacity = opacity

    def selected_item__set(self, index):
        if index < 0 or index >= len(self.widgets):
            return

        half_size = (self.visible_range_size-1.0)/2.0
        self._selected_item = index

        if self.animated:
            self._animated.visible_range_start = index
        else:
            self.visible_range_start = index

if __name__ == "__main__":
    import pgm
    import gobject
    import gst
    import glob, sys
    from pgm.graph.text import Text
    from pgm.graph.image import Image

    def create_text(label):
        txt = Text()
        txt.label = label
        txt.font_family = "Bitstream DejaVu"
        txt.font_height = 0.225
        txt.fg_color = (255, 255, 255, 255)
        txt.bg_color = (255, 0, 0, 255)
        txt.ellipsize = pgm.TEXT_ELLIPSIZE_END
        txt.visible = True
        return txt

    def create_img(img_file):
        img = Image()
        img.set_from_file(img_file, 512)
        img.fg_color = (255, 255, 255, 255)
        img.bg_color = (100, 200, 100, 155)
        img.bg_color = (0, 0, 0, 0)
        img.visible = True
        return img

    def create_reflection(master_img):
        img = Image()
        img.set_from_image(master_img)
        img.fg_color = (255, 255, 255, 255)
        img.bg_color = (100, 100, 200, 155)
        img.bg_color = (0, 0, 0, 0)
#        img.width = -master_img.width
        img.height = master_img.height
        img.opacity = 30
#        img.x += master_img.width
        img.layout = pgm.IMAGE_SCALED
        img.y += master_img.height
        img.alignment = pgm.IMAGE_TOP
        img.visible = True
        return img

    def create_video(video_uri):
        img = Image()
        img.fg_color = (255, 255, 255, 255)
        img.bg_color = (0, 0, 0, 0)
        img.alignment = pgm.IMAGE_LEFT
        img.visible = True

        # GStreamer pipeline setup
        pipeline = gst.element_factory_make('playbin')
        sink = gst.element_factory_make('pgmimagesink')
        pipeline.set_property('uri', video_uri)
        pipeline.set_property('video-sink', sink)
        sink.set_property('image', img)
        pipeline.set_state(gst.STATE_PLAYING)

        return img


    def on_key_press(viewport, event, gl, widget):
        if event.type == pgm.KEY_PRESS:
            # quit on q or ESC
            if event.keyval == pgm.keysyms.q or \
               event.keyval == pgm.keysyms.Escape:
                pgm.main_quit()
            
            elif event.keyval == pgm.keysyms.f:
                viewport.fullscreen = not viewport.fullscreen

            elif event.keyval == pgm.keysyms.b:
                widget.selected_item = 0

            elif event.keyval == pgm.keysyms.g:
                widget.selected_item = len(widget) - 1

            elif event.keyval == pgm.keysyms.h:
                widget._animated.visible_range_size += 1

            elif event.keyval == pgm.keysyms.n:
                widget._animated.visible_range_size -= 1

            elif event.keyval == pgm.keysyms.Down or \
                 event.keyval == pgm.keysyms.Right:
                widget.selected_item += 1

            elif event.keyval == pgm.keysyms.Up or \
                 event.keyval == pgm.keysyms.Left:
                widget.selected_item -= 1

            elif event.keyval == pgm.keysyms.space:
                #widget.insert(0, create_text("T"))
                def test():
#                    img = create_img("/home/kaleo/dev/pigment/examples/pictures/fluendo.png")
                    """
                    list_widget.insert(0, img)
                    list_widget.pop(len(list_widget)-1)
                    """
                    img = list_widget.pop(0)
                    list_widget.append(img)
                    return True
                gobject.timeout_add(1000, test)
#                list_widget.append(img)

            # remove the currently selected item
            elif event.keyval == pgm.keysyms.Return:
                widget.pop(widget.selected_item)

    def on_delete(viewport, event):
        pgm.main_quit()


    # OpenGL viewport creation
    factory = pgm.ViewportFactory('opengl')
    gl = factory.create()
    gl.title = 'Circular list widget'
    gl.size = (640, 480)

    # Canvas and image drawable creation
    canvas = pgm.Canvas()

    # Bind the canvas to the OpenGL viewport
    gl.set_canvas(canvas)
    gl.show()

    list_widget = CircularList()
    list_widget.position = (0.5, 0.0, 0.0)
    list_widget.width = 3.0
    list_widget.height = 3.0
    list_widget.visible_range_size = 8
    list_widget.visible = True
    list_widget.canvas = canvas

    files = sys.argv[1:]
    for file in files:
        image = create_img(file)
        list_widget.append(image)
        """
        # reflection code
        image.alignment = pgm.IMAGE_BOTTOM
        reflection = create_reflection(image)
        g = Group(canvas, pgm.DRAWABLE_MIDDLE)
        g.add(image)
        g.add(reflection)
        g.visible = True
        list_widget.append(g)
        """
    
    # Let's start a mainloop
    gl.connect('key-press-event',
               on_key_press,
               gl,
               list_widget)
    gl.connect('delete-event', on_delete)
    pgm.main()



# -*- Mode: Python; py-indent-offset: 4 -*-
'''Simple module for extracting GNOME style doc comments from C
sources, so I can use them for other purposes.'''

import sys, os, string, re

__all__ = ['extract']

comment_line_lead = re.compile(r'^\s*\*\s*')
funcname_pat = re.compile(r'^(\w+)\s*:?')
sectionname_pat = re.compile(r'^SECTION\s*:\s*(\w+)\s*$')
comment_start_pat = re.compile(r'^\s*/\*\*\s')
comment_end_pat = re.compile(r'^\s*\*+/')

newline_pat = re.compile(r'^\s*\n\s*$')
return_pat = re.compile(r'^@?(returns:|return\s+value:|returns\s*)(.*\n?)$',
                        re.IGNORECASE)
param_pat = re.compile(r'^@(\S+)\s*:(.*\n?)$')

class Doc:
    name = None
    def __init__(self, name=None):
        self.name = name

    @staticmethod
    def new_doc(line):
        # it's important to check sectionname_pat first, since it's a more
        # specific case of funcname_pat.
        for regexp,DocType in [(sectionname_pat, ClassDoc),
                               (funcname_pat, FunctionDoc)]:
            match = regexp.match(line)
            if match:
                return DocType(match.group(1))
        return None

    def _strip_comment_lead(self, line):
        return comment_line_lead.sub('', line)

    def parse_line(self, line):
        """Parses a line and fills the Doc accordingly, returns False if we
        reached the end of the comment"""
        match = comment_end_pat.match(line)
        if match:
            return False
        clean_line = self._strip_comment_lead(line)
        self.do_parse_line(clean_line)
        return True

    def do_parse_line(self, line):
        raise NotImplementedError

    def set_name(self, name):
        self.name = name

    def has_enough(self):
        return self.name != None


class FunctionDoc(Doc):
    #states
    IN_DESCRIPTION = 1
    IN_RETURN = 2
    IN_PARAM = 3
    def __init__(self, name=None):
        self.params = []
        self.description = ''
        self.ret = ''
        self.current_state = self.IN_DESCRIPTION
        Doc.__init__(self, name)

    def add_param(self, name, description):
        if name == '...':
            name = 'Varargs'
        self.params.append((name, description.strip()))
    def append_to_last_param(self, extra):
        self.params[-1] = (self.params[-1][0], self.params[-1][1] + extra)
    def append_to_named_param(self, name, extra):
        for i in range(len(self.params)):
            if self.params[i][0] == name:
                self.params[i] = (name, self.params[i][1] + extra)
                return
        # fall through to adding extra parameter ...
        self.add_param(name, extra)
    def append_description(self, extra):
        self.description = self.description + extra
    def append_return(self, extra):
        self.ret = self.ret + extra

    def get_param_description(self, name):
        for param, description in self.params:
            if param == name:
                return description
        else:
            return ''

    def set_state_from_line(self, line):
        """Changes .current_state if the (comment) line requires it. Returns
        the match that triggered the change or None."""
        for pat, state in [ (newline_pat, self.IN_DESCRIPTION),
                            (return_pat, self.IN_RETURN),
                            (param_pat, self.IN_PARAM)]:
            match = pat.match(line)
            if match:
                self.current_state = state
                return match

    def do_parse_line(self, line):
        param = None
        if not line: line = '\n'

        match = self.set_state_from_line(line)

        if self.current_state == self.IN_PARAM:
            if match:
                param = match.group(1)
                desc = match.group(2)
                self.add_param(param, desc)
            else:
                self.append_to_last_param(line)
        elif self.current_state == self.IN_RETURN:
            if match:
                return_start = match.group(1)
                self.ret = match.group(2)
                self.description = self.description + return_start + self.ret
            else:
                self.append_return(line)
        elif self.current_state == self.IN_DESCRIPTION:
            if match: #newline
                pass
            else:
                if line[:12] == 'Description:':
                    line = line[12:]
                self.append_description(line)
        else:
           print >> sys.stderr, "Wooops!"

class ClassDoc(Doc):
    def __init__(self, name=None):
        self.name = None
        self.short_description = None
        self.see_also = []
        Doc.__init__(self, name)

    def set_name(self, name):
        self.name = name

    def set_short_description(self, desc):
        self.short_description = desc.strip()

    def add_see_also_line(self, line):
        self.see_also += [name.strip(' #.') for name in line.split(',')]

    def do_parse_line(self, line):
        match = param_pat.match(line)
        if match:
            name = match.group(1)
            desc = match.group(2)
            if name == 'short_description':
                self.set_short_description(desc)
            elif name == 'see_also':
                self.add_see_also_line(desc)




def parse_file(fp, func_docs, class_docs):
    line = fp.readline()
    in_comment_block = False
    while line:
        if not in_comment_block:
            if comment_start_pat.match(line):
                in_comment_block = True
                continue
            else:
                line = fp.readline()
                continue

        # inside a comment block, and not the end of the block ...
        line = comment_line_lead.sub('', line)

        doc = Doc.new_doc(line)
        if doc:
            ret = True
            print >> sys.stderr, "    name=", doc.name
            line = fp.readline() # we skip the line giving the name
            while in_comment_block:
                ret = doc.parse_line(line)
                if not ret:
                    in_comment_block = False
                line = fp.readline()
                if not line:
                    in_comment_block = False
                    break
            if isinstance(doc, FunctionDoc):
                func_docs[doc.name] = doc
            elif isinstance(doc, ClassDoc):
                class_docs[doc.name] = doc
            else:
                raise NotImplementedError
        line = fp.readline()

def parse_dir(dir, func_docs, class_docs):
    for file in os.listdir(dir):
        if file in ('.', '..'): continue
        path = os.path.join(dir, file)
        if os.path.isdir(path):
            parse_dir(path, func_docs, class_docs)
        if len(file) > 2 and file[-2:] == '.c':
            parse_file(open(path, 'r'), func_docs, class_docs)

def extract(dirs, func_docs=None, class_docs=None):
    if not func_docs: func_docs = {}
    if not class_docs: class_docs = {}
    for dir in dirs:
        parse_dir(dir, func_docs, class_docs)
    return (func_docs, class_docs)

tmpl_section_pat = re.compile(r'^<!-- ##### (\w+) (\w+) ##### -->$')
def parse_tmpl(fp, doc_dict):
    cur_doc = None

    line = fp.readline()
    while line:
        match = tmpl_section_pat.match(line)
        if match:
            cur_doc = None  # new input shouldn't affect the old doc dict
            sect_type = match.group(1)
            sect_name = match.group(2)

            if sect_type == 'FUNCTION':
                cur_doc = doc_dict.get(sect_name)
                if not cur_doc:
                    cur_doc = FunctionDoc()
                    cur_doc.set_name(sect_name)
                    doc_dict[sect_name] = cur_doc
        elif line == '<!-- # Unused Parameters # -->\n':
            cur_doc = None # don't worry about unused params.
        elif cur_doc:
            if line[:10] == '@Returns: ':
                if string.strip(line[10:]):
                    cur_doc.append_return(line[10:])
            elif line[0] == '@':
                pos = string.find(line, ':')
                if pos >= 0:
                    cur_doc.append_to_named_param(line[1:pos], line[pos+1:])
                else:
                    cur_doc.append_description(line)
            else:
                cur_doc.append_description(line)

        line = fp.readline()

def extract_tmpl(dirs, doc_dict=None):
    if not doc_dict: doc_dict = {}
    for dir in dirs:
        for file in os.listdir(dir):
            if file in ('.', '..'): continue
            path = os.path.join(dir, file)
            if os.path.isdir(path):
                continue
            if len(file) > 2 and file[-2:] == '.sgml':
                parse_tmpl(open(path, 'r'), doc_dict)
    return doc_dict



# -*- mode: python; coding: utf-8 -*-
#
# Pigment Python tools
#
# Copyright © 2006, 2007, 2008 Fluendo Embedded S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

import pgm
from pgm.graph.group import Group
from pgm.graph.image import Image
from pgm.timing import implicit
from pgm.widgets import const
from pgm.utils import maths

import math
import gobject
import time as mod_time

class List(Group):

    __gsignals__ = {
        'child-clicked': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
                (gobject.TYPE_INT,)),
        'selected-item-changed': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
                (gobject.TYPE_INT,)),
        'scrolled': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_BOOLEAN,
                (gobject.TYPE_FLOAT, gobject.TYPE_FLOAT, gobject.TYPE_FLOAT,
                pgm.ScrollDirection, gobject.TYPE_UINT)),
    }
    
    # time between two drag events in milliseconds
    drag_motion_resolution = 30

    # amount of movement needed to activate dragging in canvas coordinates
    drag_threshold = 0.05

    # number of items that are not yet visible that should be preloaded
    preloaded = 5

    # animation mode of the widget: animated or not animated
    animated = True

    def __init__(self, width=1.0, height=2.0, visible_range_size=7,
                 orientation=const.VERTICAL):
        super(List, self).__init__()

        self.widgets = []

        self._selected_item = 0
        self._visible_range_start = 0.0
        self._orientation = orientation
        self._preloaded_widgets = []

        self._width = width
        self._height = height

        self._last_drag_motion = 0
        self.visible_range_size = visible_range_size

        # mouse support
        self._drag_zone = Image()
        self._drag_zone.bg_color = (255, 0, 0, 0)
        self._drag_zone.size = (width, height)
        self._drag_zone.visible = True
        Group.add(self, self._drag_zone)
        self._drag_zone.connect("drag-begin", self._drag_begin)
        self._drag_zone.connect("drag-motion", self._drag_motion)
        self._drag_zone.connect("drag-end", self._drag_end)
        self._drag_zone.connect("clicked", self._clicked)
        self._drag_zone.connect("double-clicked", self._double_clicked)
        self._drag_zone.connect("scrolled", self._scrolled)
        
        # animation support
        attributes = ("visible_range_size", "visible_range_start")
        self._animated = implicit.AnimatedObject(self, attributes)
        self._animated.setup_next_animations(duration=500,
                                             transformation=implicit.DECELERATE)
        self._animated.mode = implicit.REPLACE

        self._dragging = False
        self._drag_accum = 0

    def _scrolled(self, drawable, x, y, z, direction, time):
        return self.emit('scrolled', x, y, z, direction, time)

    def _drag_begin(self, drawable, x, y, z, button, time):
        self._dragging = True
        return self.emit('drag-begin', x, y, z, button, time)

    def _drag_motion(self, drawable, x, y, z, button, time):
        time_since_last = time - self._last_drag_motion
        if time_since_last > self.drag_motion_resolution:
            self._last_drag_motion = time
            return self.emit('drag-motion', x, y, z, button, time)

    def _drag_end(self, drawable, x, y, z, button, time):
        self._dragging = False
        return self.emit('drag-end', x, y, z, button, time)

    def _clicked(self, drawable, x, y, z, button, time):
        if self._dragging and self._drag_accum > self.drag_threshold:
            return True

        return self.emit('clicked', x, y, z, button, time)

    def _double_clicked(self, drawable, x, y, z, button, time):
        if self._dragging:
            return False

        return self.emit('double-clicked', x, y, z, button, time)

    def width__get(self):
        return self._width

    def width__set(self, width):
        self._width = width
        self._drag_zone.width = width

        self._widget_width = self.compute_width(0)
        self._layout()

    def height__get(self):
        return self._height

    def height__set(self, height):
        self._height = height
        self._drag_zone.height = height

        self._widget_height = self.compute_height(0)
        self._layout()

    def size__get(self):
        return (self._width, self._height)

    def size__set(self, size):
        self.width = size[0]
        self.height = size[1]

    def visible_range_size__get(self):
        return self._visible_range_size

    def visible_range_size__set(self, visible_range_size):
        self._visible_range_size = visible_range_size

        self._widget_height = self.compute_height(0)
        self._widget_width = self.compute_width(0)
        self._layout()

    def compute_height(self, index):
        if self._orientation == const.VERTICAL:
            return self._height / self._visible_range_size
        elif self._orientation == const.HORIZONTAL:
            return self._height

    def compute_width(self, index):
        if self._orientation == const.VERTICAL:
            return self._width
        elif self._orientation == const.HORIZONTAL:
            return self._width / self._visible_range_size

    def compute_x(self, index):
        if self._orientation == const.VERTICAL:
            return 0.0
        elif self._orientation == const.HORIZONTAL:
            return index * self._widget_width

    def compute_y(self, index):
        if self._orientation == const.VERTICAL:
            return index * self._widget_height
        elif self._orientation == const.HORIZONTAL:
            return 0.0

    def compute_z(self, index):
        return 0.0

    def compute_opacity(self, index):
        full_opacity = 255
        shaded_opacity = 100
        invisible_opacity = -80

        # make the transformation symmetrical
        if index <= self._visible_range_size/2.0:
            # beginning of the visible items case
            index = index
            start = self._visible_range_start
        else:
            # end of the visible items case
            index = self._visible_range_size - index - 1
            start = len(self) - self._visible_range_size -\
                    self._visible_range_start

        if start <= 0 or index >= 1 or self._visible_range_size <= 1.0:
            opacity = full_opacity
        elif index >= 0:
            opacity = maths.lerp(shaded_opacity, full_opacity, index)
        elif index < 0:
            if start >= 1:
                opacity = maths.lerp(invisible_opacity, shaded_opacity, index+1)
            else:
                opacity = maths.lerp(invisible_opacity, full_opacity, index+1)

        opacity = max(min(255, opacity), 0)

        return opacity

    def compute_zoom_width(self, index):
        return 1.0

    def compute_zoom_height(self, index):
        return 1.0

    def visible_range_start__set(self, visible_range_start):
        self._visible_range_start = visible_range_start
        self._layout()

    def visible_range_start__get(self):
        return self._visible_range_start

    def selected_item__get(self):
        return self._selected_item

    def selected_item__set(self, index):
        self._stop_deceleration()
        index = maths.clamp(index, 0, len(self.widgets)-1)
        index = int(round(index))

        half_size = (self.visible_range_size-1.0)/2.0
        prev_selected = self._selected_item
        self._selected_item = index
        if index <= half_size or len(self.widgets) <= self.visible_range_size:
            visible_range_start = 0.0
        elif index >= len(self.widgets)-half_size:
            visible_range_start = len(self.widgets)-self.visible_range_size
        else:
            visible_range_start = index-half_size

        if self.animated:
            self._animated.visible_range_start = visible_range_start
        else:
            self.visible_range_start = visible_range_start

        if prev_selected != index:
            self.emit('selected-item-changed', index)

    def range_start_to_selected(self):
        half_size = (self.visible_range_size-1.0)/2.0
        selected = self.visible_range_start + half_size
        selected = int(round(selected))
        selected = maths.clamp(selected, 0, len(self.widgets)-1)
        return selected

    def layout_widget(self, widget, position):
        width = self._widget_width*self.compute_zoom_width(position)
        height = self._widget_height*self.compute_zoom_height(position)

        # update widget properties
        x = self.compute_x(position)-(width-self._widget_width)/2.0
        y = self.compute_y(position)-(height-self._widget_height)/2.0
        z = self.compute_z(position)
        widget.position = (x, y, z)
        widget.size = (width, height)
        widget.opacity = self.compute_opacity(position)

    def visible__set(self, value):
        old_value = self.visible
        super(List, self).visible__set(value)
        if value and not old_value:
            self._layout()

    def _layout(self):
        if not self.visible:
            return

        # widgets that will be unloaded once the layout is done
        to_unload = set(self._preloaded_widgets)

        # preloads widgets at the edge of the visible range
        # update properties for visible widgets only
        inf = int(math.floor(self._visible_range_start))
        sup = int(math.ceil(self._visible_range_start+self._visible_range_size))

        inf_preloaded = inf - self.preloaded
        sup_preloaded = sup + self.preloaded

        inf_preloaded = max(0, inf_preloaded)
        sup_preloaded = min(len(self.widgets), sup_preloaded)

        inf = max(0, inf)
        sup = min(len(self.widgets), sup)

        self._preloaded_widgets = self.widgets[inf_preloaded:sup_preloaded]
        for i, widget in enumerate(self._preloaded_widgets):
            i += inf_preloaded
            position = -self._visible_range_start + i

            # update to_unload accordingly
            if widget in to_unload:
                to_unload.remove(widget)
            else:
                self.load_item(i)

            if i >= inf and i < sup:
                # widget is in the visible range and therefore needs to get
                # its properties set (position, size, etc.)
                self.layout_widget(widget, position)

                # adding the widget to the list group so that it's eventually
                # displayed
                # most of the time it is not necessary to add the widget to
                # the Group since it's already there
                Group.add(self, widget)

                # FIXME: this should not be necessary but for some unknown
                # reason some items do not show up when scrolling quickly if
                # this assignment is removed
                widget.visible = True
            else:
                Group.remove(self, widget)

        # unload widgets that were loaded before that now must be unloaded
        for widget in to_unload:
            Group.remove(self, widget)
            self.unload_item(widget)

    def load_item(self, index):
        pass

    def unload_item(self, widget):
        pass

    def is_widget_visible(self, index):
        inf = int(math.floor(self._visible_range_start))
        sup = int(math.ceil(self._visible_range_start+self._visible_range_size))
        return index >= inf and index < sup

    def orientation__set(self, orientation):
        self._orientation = orientation
        self.visible_range_size = self._visible_range_size

    def orientation__get(self):
        return self._orientation

    def __len__(self):
        return len(self.widgets)

    def insert(self, index, widget):
        if index < 0 or index > len(self.widgets):
            raise IndexError

        # do not add the same widget twice
        if widget in self.widgets:
            return

        widget.visible = True

        self.widgets.insert(index, widget)
        if not self._animated.is_animated("visible_range_start"):
            sup = int(self._visible_range_start+self._visible_range_size+1.0)
            if index <= sup:
                self._layout()

    def append(self, widget):
        self.insert(len(self.widgets), widget)

    def remove(self, widget):
        try:
            # if the widget is part of the List
            index = self.widgets.index(widget)
        except ValueError:
            # if the widget is just part of the Group
            Group.remove(self, widget)
        else:
            self.pop(index)

    def pop(self, index):
        if index < 0 or index >= len(self.widgets):
            raise IndexError

        if self.selected_item == index == len(self.widgets)-1:
            self.selected_item -= 1

        widget = self.widgets.pop(index)
        Group.remove(self, widget)
        if not self._animated.is_animated("visible_range_start"):
            sup = int(self._visible_range_start+self._visible_range_size+1.0)
            if index <= sup:
                self._layout()

        return widget

    def empty(self):
        for widget in self.widgets:
            Group.remove(self, widget)
        self.widgets[:] = []
        self._selected_item = 0
        self._layout()
        
    def __getitem__(self, index):
        return self.widgets[index]

    def do_clicked(self, x, y, z, button, time):
        if self._orientation == const.VERTICAL:
            visible_selected = \
                    (y - self.y) / (self.height / self.visible_range_size)
        else:
            visible_selected = \
                    (x - self.x) / (self.width / self.visible_range_size)

        child_clicked = int(self.visible_range_start + visible_selected)

        self.emit('child-clicked', child_clicked)
        return True

    def do_double_clicked(self, x, y, z, button, time):
        return True

    def do_drag_begin(self, x, y, z, button, time):
        self._initial = (x, y, time)
        self._animated.stop_animations()
        self._stop_deceleration()

        self._drag_accum = 0

        self.speed = 0.0

        return True

    def do_drag_motion(self, x, y, z, button, time):
        motion = 0

        if self._orientation == const.VERTICAL:
            motion = y - self._initial[1]
            self.visible_range_start -= motion / self._widget_height
        elif self._orientation == const.HORIZONTAL:
            motion = x - self._initial[0]
            self.visible_range_start -= motion / self._widget_width

        time_delta = time - self._initial[2]
        if time_delta != 0:
            self.speed = motion/time_delta*1000.0

        self._initial = (x, y, time)
        self._drag_accum += abs(motion)

        return True

    def do_drag_end(self, x, y, z, button, time):
        if self._drag_accum > self.drag_threshold:
            self._current_time = mod_time.time()
            self._deceleration_source = gobject.timeout_add(17, self._decelerate)
        return True

    def do_child_clicked(self, index):
        self.selected_item = index
        return True

    deceleration = 8.0
    def _decelerate(self):
        self._previous_time = self._current_time
        self._current_time = mod_time.time()
        delta = self._current_time - self._previous_time

        if self.speed > 0.0:
            self.speed -= self.deceleration*delta
            if self.speed > 0.0:
                self.visible_range_start -= self.speed*delta

                # block the movement if it reaches the first item
                if self.range_start_to_selected() > 0:
                    return True

        elif self.speed < 0.0:
            self.speed += self.deceleration*delta
            if self.speed < 0.0:
                self.visible_range_start -= self.speed*delta

                # block the movement if it reaches the last item
                if self.range_start_to_selected() < len(self.widgets)-1:
                    return True

        self.selected_item = self.range_start_to_selected()
        return False

    def _stop_deceleration(self):
        try:
            gobject.source_remove(self._deceleration_source)
        except AttributeError:
            pass

if __name__ == "__main__":
    import pgm
    import gobject
    import gst
    import glob, sys
    from pgm.graph.text import Text
    from pgm.graph.image import Image

    def create_text(label):
        txt = Text()
        txt.label = label
        txt.font_family = "Bitstream DejaVu"
        txt.font_height = 0.225
        txt.fg_color = (255, 255, 255, 255)
        txt.bg_color = (255, 0, 0, 255)
        txt.ellipsize = pgm.TEXT_ELLIPSIZE_END
        txt.visible = True
        return txt

    def create_img(img_file):
        img = Image()
        img.set_from_file(img_file, 512)
        img.fg_color = (255, 255, 255, 255)
        img.bg_color = (100, 200, 100, 155)
        img.bg_color = (0, 0, 0, 0)
        img.visible = True
        return img

    def create_reflection(master_img):
        img = Image()
        img.set_from_image(master_img)
        img.fg_color = (255, 255, 255, 255)
        img.bg_color = (100, 100, 200, 155)
        img.bg_color = (0, 0, 0, 0)
#        img.width = -master_img.width
        img.height = master_img.height
        img.opacity = 30
#        img.x += master_img.width
        img.layout = pgm.IMAGE_SCALED
        img.y += master_img.height
        img.alignment = pgm.IMAGE_TOP
        img.visible = True
        return img

    def create_video(video_uri):
        img = Image()
        img.fg_color = (255, 255, 255, 255)
        img.bg_color = (0, 0, 0, 0)
        img.alignment = pgm.IMAGE_LEFT
        img.visible = True

        # GStreamer pipeline setup
        pipeline = gst.element_factory_make('playbin')
        sink = gst.element_factory_make('pgmimagesink')
        pipeline.set_property('uri', video_uri)
        pipeline.set_property('video-sink', sink)
        sink.set_property('image', img)
        pipeline.set_state(gst.STATE_PLAYING)

        return img


    def on_key_press(viewport, event, widget):
        if event.type == pgm.KEY_PRESS:
            # quit on q or ESC
            if event.keyval == pgm.keysyms.q or \
               event.keyval == pgm.keysyms.Escape:
                pgm.main_quit()
            
            elif event.keyval == pgm.keysyms.f:
                viewport.fullscreen = not viewport.fullscreen

            elif event.keyval == pgm.keysyms.b:
                widget.selected_item = 0

            elif event.keyval == pgm.keysyms.g:
                widget.selected_item = len(widget) - 1

            elif event.keyval == pgm.keysyms.h:
                #widget.visible_range_size += 2
                widget._animated.visible_range_size += 2

            elif event.keyval == pgm.keysyms.n:
                #widget.visible_range_size -= 2
                widget._animated.visible_range_size -= 2

            elif event.keyval == pgm.keysyms.s:
                if list_widget.orientation == const.VERTICAL:
                    list_widget.orientation = const.HORIZONTAL
                elif list_widget.orientation == const.HORIZONTAL:
                    list_widget.orientation = const.VERTICAL

            elif event.keyval == pgm.keysyms.Down or \
                 event.keyval == pgm.keysyms.Right:
                widget.selected_item += 1

            elif event.keyval == pgm.keysyms.Up or \
                 event.keyval == pgm.keysyms.Left:
                widget.selected_item -= 1

            elif event.keyval == pgm.keysyms.space:
                #widget.insert(0, create_text("T"))
                def test():
#                    img = create_img("/home/kaleo/dev/pigment/examples/pictures/fluendo.png")
                    """
                    widget.insert(0, img)
                    widget.pop(len(widget)-1)
                    """
                    img = widget.pop(0)
                    widget.append(img)
                    return True
                gobject.timeout_add(1000, test)
#                widget.append(img)

            # remove the currently selected item
            elif event.keyval == pgm.keysyms.Return:
                widget.pop(widget.selected_item)

    def on_delete(viewport, event):
        pgm.main_quit()


    # OpenGL viewport creation
    factory = pgm.ViewportFactory('opengl')
    gl = factory.create()
    gl.title = 'List widget'

    # Canvas and image drawable creation
    canvas = pgm.Canvas()

    # Bind the canvas to the OpenGL viewport
    gl.set_canvas(canvas)
    gl.show()

    widget = List()
    widget.position = (0.5, 0.5, 0.0)
    widget.orientation = const.HORIZONTAL
    widget.width = 3.0
    widget.height = 2.0
    widget.visible_range_size = 7
    widget.visible = True
    widget.canvas = canvas

    files = sys.argv[1:]
    for file in files:
        image = create_img(file)
        widget.append(image)
        """
        # reflection code
        image.alignment = pgm.IMAGE_BOTTOM
        reflection = create_reflection(image)
        g = Group(canvas, pgm.DRAWABLE_MIDDLE)
        g.add(image)
        g.add(reflection)
        g.visible = True
        widget.append(g)
        """

    # Let's start a mainloop
    gl.connect('key-press-event',
               on_key_press,
               widget)
    gl.connect('delete-event', on_delete)
    pgm.main()



# -*- mode: python; coding: utf-8 -*-
#
# Pigment Python binding timing example
#
# Copyright © 2006, 2007, 2008 Fluendo Embedded S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# Author: Benjamin Kampmann <benjamin@fluendo.com>

import sys, gobject, math, os

class BenchmarkResult(object):
    """
    The BenchmarkResult contains the data that was found out during the
    process of benchmarking.
    @ivar hardware_acceleration:    whether hardware acceleration is enabled
    @type hardware_acceleration:    bool
    @ivar shaders:                  whether shaders are found
    @type shaders:                  bool
    @ivar size:                     the size of the screen in pixels
    @type size:                     tuple of two int
    @ivar frame_rates:              the frame rates that got captures
    @type frame_rates:              list
    @ivar max_rate:                 the maximum frame rate, that has happened
    @type max_rate:                 int
    @ivar min_rate:                 the minimum frame rate, that has happened
    @type min_rate:                 int
    @ivar average:                  the average frame rate
    @type average:                  int
    @ivar median:                   the median frame rate
    @type median:                   int
    @ivar upper_median:             the upper median
    @type upper_median:             int
    @ivar lower_median:             the lower median
    @type lower_median:             int
    """
    def __init__(self):
        self.hardware_acceleration = False
        self.shaders = False
        self.size = (0,0)
        self.max_rate = 0
        self.min_rate = 0
        self.average = 0
        self.lower_median = 0
        self.median = 0
        self.upper_median = 0
        self.frame_rates = []

OPACITY_CHANGE = 12

try:
    import pgm
except ImportError:
    print "You need to install pigment"
    print "or run this script from the pigment-uninstalled-shell"
    sys.exit(0)

class Benchmark(object):
    """
    Do a simple Benchmark. You can set some parameters to influence the
    behaviour of the benchmark. After the values got set, you should run
    L{start} to start the benchmark. After the benchmarking is done the
    L{callback} gets called.

    B{Attention}: At runtime of this class certain enviroment-variables get
    set to get a certain behaviour of pigment. Please wait until the
    callback-call was done to run your own pigment application in the original
    enviroment.

    @ivar picture_path: the path to the image to load and let it rotate
    @type picture_path: str
    @ivar frames:       the number of frames to catch (default: 100)
    @type frames:       int
    @ivar callback:     to call after the benchmark is done. the first argument
                        will be the BenchmarkResult containing the values
    @type callback:     callable
    @ivar transformation: the transformation matrix to do to the image
    @ivar text:         the pgm-text. If you want to display another text, you could
                        do::
                            benchmark.text.label = "My new Text!"
    @ivar opacity_change: the step the opacity of the text should be changed
                          at any iteration
    """

    TEST_PLUGIN = 'opengl'
    """
    The plugin to test. Needs to be set BEFORE creating the Object
    """

    def __init__(self):
        self._vblank = os.environ.get('PGM_GL_VBLANK', '1')
        # don't sync to vblank for this test!
        os.environ['PGM_GL_VBLANK'] = '0'
        self.result = BenchmarkResult()

        self._fps = 0
        # remove any frame per second guessing
        if os.environ.has_key('PGM_GL_FPS'):
            self._fps = os.environ['PGM_GL_FPS']
            del os.environ['PGM_GL_FPS']

        # OpenGL viewport creation
        self.gl = pgm.viewport_factory_make(self.TEST_PLUGIN)

        # pgm was found!
        self.result.has_pgm = True

        self.gl.title = 'Benchmark'
        self.gl.fullscreen = True
        self.gl.decorated = True
        self.gl.cursor = pgm.VIEWPORT_NONE

        # check for hardware accelartion
        for cap in self.gl.caps_mask:
            if 'PGM_VIEWPORT_HARDWARE_ACCELERATION' in cap.value_names:
                self.result.hardware_acceleration = True
                break

        # check for shaders
        for format in self.gl.pixel_formats:
            if 'PGM_IMAGE_I420' in format.value_names:
                self.result.shaders = True
                break

        cvs = pgm.Canvas()
        self.gl.set_canvas(cvs)

        self.callback = None

    def _clean_up(self):
        os.environ['PGM_GL_VBLANK'] = self._vblank

        if self._fps:
            os.environ['PGM_GL_FPS'] = self._fps
        self.gl.set_canvas(None)
        self.gl.visible = False
        self.gl.hide()
        del self.gl
        if self.callback:
            self.callback(self.result)


    def start(self):
#        self.gl.show()
        gobject.timeout_add(100, self._clean_up)

    def _resulting(self):
        # calculate average
        total = 0
        result_num = len(self.result.frame_rates)
        for fps in self.result.frame_rates:
            total += fps

        average = total / result_num

        self.result.frame_rates.sort()
        # max
        max_rate = self.result.frame_rates[-1]
        # min
        min_rate = self.result.frame_rates[0]

        # middle
        middle_pos = result_num / 2
        quarter_pos = middle_pos / 2

        median = self.result.frame_rates[middle_pos]
        lower_median = self.result.frame_rates[quarter_pos]
        upper_median = self.result.frame_rates[middle_pos+quarter_pos]

        self.result.size = self.gl.size
        self.result.max_rate = max_rate
        self.result.min_rate = min_rate
        self.result.average = average
        self.result.lower_median = lower_median
        self.result.median = median
        self.result.upper_median = upper_median
        self._clean_up()

def main(args):

    try:
        frames = int(args[0])
    except:
        frames = 100

    b = Benchmark()

    def print_result(data):
        print "HW ac found:", data.hardware_acceleration
        pgm.stop()


    b.frames = frames
    b.start()
    b.callback = print_result
    pgm.main()

if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



# -*- mode: python; coding: utf-8 -*-
#
# Pigment Python tools
#
# Copyright © 2006, 2007, 2008 Fluendo Embedded S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

from pgm.graph.group import Group
from pgm.graph.image import Image
from pgm.timing import implicit
from pgm.widgets import const
import pgm, gobject

class Scrollbar(Group):

    __gsignals__ = { 
        'index-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
                           (gobject.TYPE_INT,))
        }


    def __init__(self, orientation=const.VERTICAL, items_number=0,
                 thickness=0.1,
                 cursor=None, background=None, spacing=None
                 ):
        """
        @keyword orientation:  bar orientation, const.VERTICAL by default
        @type orientation:     int
        @keyword items_number: how many cursors the bar can scroll
        @type items_number:    int
        @keyword cursor_color: color of the bar cursor, white by default
        @type cursor_color:    4-int tuple
        @keyword bg_color:     color of the bar background, black by default
        @type bg_color:        4-int tuple
        """
        Group.__init__(self)
        self._items_number = items_number
        self._current_index = 0
        self._cur_size = None
        self._cur_pos = None
        self._orientation = orientation
        self._thickness = thickness
        self._background = None
        self._cursor = None
        self._animated_cursor = None

        if spacing is None:
            spacing = 0
            
        self._spacing = spacing
        
        if not cursor:
            # cursor, white by default
            cursor_color = (255,255,255,255)
            cursor = Image()
            cursor.bg_color= cursor_color
            
        if not background:
            # background of the bar, black by default
            bg_color = (0,0,0,255)
            background = Image()
            background.bg_color = bg_color

        self.background = background        
        self.cursor = cursor
        self.visible = True

        # animation support
        self._animated = implicit.AnimatedObject(self)
        self._animated.setup_next_animations(duration=500, resolution=5,
                                             transformation=implicit.DECELERATE)
        self._animated.mode = implicit.REPLACE


    def background__set(self, background):
        if self._background:
            self.remove(self._background)
            
        self._background = background
        self._background.visible = True
        self.add(self._background)
        self._background.size = self.size

        self._background.connect("scrolled", self._scrolled)
        
        self._update_cursor_size()
        self._update_cursor_position()
        
    def background__get(self):
        return self._background

    def cursor__set(self, cursor):
        if self._cursor:
            self.remove(self._cursor)

        self._cursor = cursor
        self._cursor.visible = True
        self.add(self._cursor)
        self._cursor.connect("drag-begin", self._drag_begin)
        self._cursor.connect("drag-motion", self._drag_motion)
        self._cursor.connect("drag-end", self._drag_end)
        self._cursor.connect("pressed", self._drag_pressed)
        
        self._update_cursor_size()
        self._update_cursor_position()

        attrs = ('cursor_index','position')
        self._animated_cursor = implicit.AnimatedObject(self._cursor, attrs)
        settings = {'duration': 300,
                    'transformation': implicit.LINEAR,
                    'resolution': 5}
        self._animated_cursor.setup_next_animations(**settings)
        self._animated_cursor.mode = implicit.REPLACE
        
    def cursor__get(self):
        return self._cursor

    def thickness__get(self):
        return self._thickness

    def thickness__set(self, thickness):
        self._thickness = thickness
        self.size = self.size

    def orientation__set(self, orientation):
        self._orientation = orientation
        self._update_cursor_size()
        self._update_cursor_position()

    def orientation__get(self):
        return self._orientation
        
    def size__set(self, size):
        size = self._apply_thickness(size)
        Group.size__set(self, size)
        self.background.size = size

        self._update_cursor_size()
        self._update_cursor_position()

    def items_number__set(self, nb):
        if nb <= 0:
            nb = 1
            
        self._items_number = nb

        if self._current_index > (nb-1):
            self._current_index = nb - 1
        
        self._update_cursor_size()
        self._update_cursor_position()

    def items_number__get(self):
        return self._items_number

    def cursor_index__set(self, position):
        """
        Set the position of the cursor. The bar is automatically new painted
        @param position: the position to set to
        @type position: something to do mathematic stuff
        """
        if position >= 0 and position < self._items_number:
            changed = position != self._current_index
            self._current_index = position
            self._update_cursor_position()
            if changed:
                self.emit('index-changed', position)
                
    def cursor_index__get(self):
        """
        returns the position of the cursor
        @return : the position of the cursor
        """
        return int(self._current_index)

    def next_item(self):
        """
        go to the next item.  The bar is automatically new painted
        """
        self.cursor_index += 1

    def previous_item(self):
        """
        go to the previous item.  The bar is automatically new painted
        """
        self.cursor_index -= 1

    def _apply_thickness(self, size):
        width, height = size
        if self._orientation == const.HORIZONTAL:
            height = self._thickness
        else:
            width = self._thickness

        size = (width, height)
        return size

    def _update_cursor_size(self):
        if self._items_number > 0:
            spacing = (self._spacing * 2)
            if self._orientation == const.VERTICAL:
                width = self.width - spacing
                height = float(self.height / self._items_number)
                if self._items_number == 1:
                    height -= spacing
            else:
                width = float(self.width / self._items_number)
                height = self.height - spacing
                if self._items_number == 1:
                    width -= spacing
                    
            size = (width, height)
        else:
            size = self.size

        self._cur_size = size
        if self._animated_cursor:
            self._animated_cursor.size = self._cur_size
        elif self._cursor:
            self._cursor.size = self._cur_size
            
    def _update_cursor_position(self):
        if self._background:
            if self._current_index == 0:
                spacing = self._spacing
            elif self._current_index == self._items_number-1:
                spacing = - self._spacing
            else:
                spacing = 0

            w, h = self._cur_size
            if self._orientation == const.VERTICAL:
                y = self.background.y + spacing + (h * self._current_index)
                x = self.background.x + self._spacing
            else:
                x = self.background.x + spacing + (w * self._current_index)
                y = self.background.y + self._spacing

            self._cur_pos = (x, y, self.z)
            if self._animated_cursor:
                self._animated_cursor.position = self._cur_pos
            elif self._cursor:
                self._cursor.position = self._cur_pos

    def _scrolled(self, widget, x, y, z, direction, time):
        if direction == pgm.SCROLL_UP:
            self.previous_item()
        else:
            self.next_item()

    def _drag_begin(self, widget, x, y, z, button, time):
        self._original_position = (x, y, z)
        #print widget
        #self._animated.stop_animations()
        
    def _drag_motion(self, widget, x, y, z, button, time):
        if self._orientation == const.VERTICAL:
            offset = y - self._original_position[1]
        else:
            offset = x - self._original_position[0]
        #print offset
        new_index = self._current_index + (offset / self._items_number)
        if new_index >= 0 and new_index < self._items_number-1:
            #self._current_index = new_index
            self.cursor_index = new_index
        self._original_position = (x, y, z)
        self._update_cursor_position()
        
    def _drag_end(self, widget, x, y, z, button, time):
        self._original_position = None
        self._current_index = round(self._current_index)
        self._update_cursor_position()
        
    def _drag_pressed(self, widget, x, y, z, button, time):
        self._drag_begin(widget, x, y, z, button, time)
        self._drag_end(widget, x, y, z, button, time)

            
if __name__ == "__main__":
    from pgm.graph.image import Image
    from pgm.graph.text import Text
    import pgm
    import os, gobject, sys
    from sliced_image import SlicedImage

    def update_bars(bars, function):
        for b in bars:
            function(b)

    def on_key_pressed(port, event, bars, loop):
        if event.keyval in (pgm.keysyms.Escape, pgm.keysyms.q):
            loop.quit()
        if event.keyval == pgm.keysyms.Up:
            update_bars(bars, lambda b: b.previous_item())
        elif event.keyval == pgm.keysyms.Down:
            update_bars(bars, lambda b: b.next_item())
        elif event.keyval == pgm.keysyms.a:
            update_bars(bars, lambda b: setattr(b, 'items_number',
                                                getattr(b, 'items_number') + 1))
        elif event.keyval == pgm.keysyms.r:
            update_bars(bars, lambda b: setattr(b, 'items_number',
                                                getattr(b, 'items_number') - 1))

    factory = pgm.ViewportFactory('opengl')
    gl = factory.create()
    gl.title = 'bar widget'

    canvas = pgm.Canvas()
    gl.set_canvas(canvas)
    gl.show()

    bar = Scrollbar(canvas, pgm.DRAWABLE_MIDDLE, items_number=12,
                    spacing=0.01)
    bar.canvas = canvas
    bar.background.bg_color = 38, 139, 200, 255
    bar.cursor.bg_color = 200, 239, 250, 155
    bar.position = (canvas.width * 0.05, canvas.height * 0.1, 0)
    bar.height = canvas.height * 0.8

    pictos = ("elisa_mockups/export/list_mode/selector/selector-left.png",
              "elisa_mockups/export/list_mode/selector/selector-right.png",
              "elisa_mockups/export/list_mode/selector/selector-body.png")
    cursor = SlicedImage(orientation=const.HORIZONTAL, *pictos)
                         
    bar2 = Scrollbar(items_number=12, orientation=const.HORIZONTAL,
                     cursor=cursor, thickness=0.5)
    bar2.canvas = canvas
    bar2.cursor.z += 1
    bar2.position = (0.5, 1, 0)
    bar2.width = canvas.width * 0.8

    pictos = ("elisa_mockups/export/list_mode/scrollbar/scrollbar-top.png",
              "elisa_mockups/export/list_mode/scrollbar/scrollbar-bottom.png",
              "elisa_mockups/export/list_mode/scrollbar/scrollbar-body.png")
    frame = SlicedImage(*pictos)
    pictos = ("elisa_mockups/export/list_mode/scrollbar/cursor-top.png",
              "elisa_mockups/export/list_mode/scrollbar/cursor-bottom.png",
              "elisa_mockups/export/list_mode/scrollbar/cursor-body.png")
    cursor = SlicedImage(*pictos)
    
    bar3 = Scrollbar(items_number=12,
                     background=frame,
                     #thickness=0.08,
                     cursor=cursor, spacing=0.01
                     )
    bar3.canvas = canvas
    bar3.position = (0.35, canvas.height * 0.1, 0)
    bar3.height = canvas.height * 0.8


    bars = [bar, bar2, bar3,]
    
    loop = gobject.MainLoop()
    gl.connect('key-press-event', on_key_pressed, bars, loop)
    loop.run()



# -*- mode: python; coding: utf-8 -*-
#
# Pigment Python binding implicit animation example
#
# Copyright © 2006, 2007, 2008 Fluendo Embedded S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# Author: Florian Boucault <florian@fluendo.com>

import sys, pgm, gobject
from pgm.timing import implicit

# Handles the key presses
def on_key_press(viewport, event, drawable, help):
    if event.keyval == pgm.keysyms.q or event.keyval == pgm.keysyms.Escape:
        pgm.main_quit()

    elif event.keyval == pgm.keysyms.Right:
        drawable.x += 0.5

    elif event.keyval == pgm.keysyms.Left:
        drawable.x -= 0.5

    elif event.keyval == pgm.keysyms.Down:
        drawable.y += 0.5

    elif event.keyval == pgm.keysyms.Up:
        drawable.y -= 0.5

    elif event.keyval == pgm.keysyms.o:
        if drawable.opacity != 0:
            drawable.opacity = 0
        else:
            drawable.opacity = 255

    elif event.keyval == pgm.keysyms.h:
        if help.opacity != 0:
            help.opacity = 0
        else:
            help.opacity = 220

# Terminate the mainloop on a delete event
def on_delete(viewport, event):
    pgm.main_quit()

def main(args):
    # OpenGL viewport creation
    gl = pgm.viewport_factory_make('opengl')
    gl.title = 'Implicit animation'

    # Canvas, image drawables creation
    cvs = pgm.Canvas()
    img = pgm.Image()

    # Bind the canvas to the OpenGL viewport
    gl.set_canvas(cvs)

    # Image properties
    img.position = (1.0, 1.5, 0.0)
    img.size = (1.0, 1.0)
    img.bg_a = 0
    img.set_from_file("examples/pictures/fluendo.png")

    # A drawable needs to be shown
    img.show()

    # Add it to the middle layer of the canvas
    cvs.add(pgm.DRAWABLE_MIDDLE, img)

    # Implicity animated image wrapping img
    animated_img = implicit.AnimatedObject(img)
    animated_img.mode = implicit.REPLACE


    # Help drawable creation
    help = pgm.Text("<b><tt>Implicit animations are great !</tt></b>\n\n"   \
                    "Keys:\n\n" \
                    "'arrows':\t move the drawable\n" \
                    "'o':\t set the drawable's opacity to 0 or 255\n" \
                    "'h':\t get rid of the help\n")

    # Help text properties tweaking
    help.position = (0.8, 1.0, 0.0)
    help.size = (2.0, 1.0)
    help.font_height = 0.1
    help.fg_color = (220, 220, 220, 255)
    help.bg_color = (20, 20, 20, 255)
    help.opacity = 0

    # A drawable needs to be shown
    help.show()

    # Add it to the middle layer of the canvas
    cvs.add(pgm.DRAWABLE_MIDDLE, help)

    # Implicity animated text wrapping help
    animated_help = implicit.AnimatedObject(help)
    animated_help.mode = implicit.REPLACE
    animated_help.opacity = 220

    # Let's start the mainloop
    gl.connect('delete-event', on_delete)
    gl.connect('key-press-event', on_key_press, animated_img, animated_help)
    gl.show()
    pgm.main()

if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



# -*- mode: python; coding: utf-8 -*-
#
# Pigment Python tools
#
# Copyright © 2006, 2007, 2008 Fluendo Embedded S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

import os, pgm, gobject
from pgm.timing import controller, modifier, keyframe
from pgm.utils import classinit


# Repeat behavior constants
FORWARD, REVERSE = 0, 1
# Repeat constant
INFINITE = -1
# Transformation constants
LINEAR, ACCELERATE, DECELERATE, SMOOTH = 5, 6, 7, 8
# Implicit mode
REPLACE, APPEND = 0, 1

# helper for implicitly animating pgm.Drawables
DRAWABLE_ATTRIBUTES = ("size",
                       "width",
                       "height",
                       "position",
                       "x",
                       "y",
                       "z",
                       "fg_color",
                       "fg_r",
                       "fg_g",
                       "fg_b",
                       "fg_a",
                       "bg_color",
                       "bg_r",
                       "bg_g",
                       "bg_b",
                       "bg_a",
                       "opacity")

class AnimatedObject(object):
    """
    This is a proxy to any object that would like some of its attributes to be
    implicitly animated when set.

    @ivar _animated_attributes: attributes of the proxied object that will be
                                implicitly animated
    @type _animated_attributes: tuple of strings
    @ivar _object:              object that is proxied and implicitly animated
    @type _object:              mutable object
    @ivar _current_controllers: controllers of the currently animated attributes
    @type _current_controllers: dict of L{pgm.timing.controller.Controller}
    @ivar _next_controllers:    controllers that will be used next time the
                                attributes are animated
    @type _next_controllers:    dict of L{pgm.timing.controller.Controller}

    """

    __metaclass__ = classinit.ClassInitMeta
    __classinit__ = classinit.build_properties
    
    def __init__(self, object, animated_attributes=DRAWABLE_ATTRIBUTES):
        """
        Initialize the controllers for the next implicit animations that will
        happen on object.

        @param object:              object to be proxied and implicitly
                                    animated
        @type object:               mutable object
        @param animated_attributes: attributes of the proxied object that will
                                    be implicitly animated
        @type animated_attributes:  tuple of strings
        """

        super(AnimatedObject, self).__init__()

        self._object = object

        self._current_controllers = {}
        self._next_controllers = {}
        self._animated_attributes = animated_attributes

        # create default controllers
        for attribute in animated_attributes:
            ctrl = controller.Controller(resolution     = 17,
                                         end_behavior   = controller.HOLD,
                                         transformation = controller.LINEAR,
                                         modifiers      = [])
            self._current_controllers[attribute] = ctrl

        self.mode = APPEND

    def static_object__get(self):
        return self._object

    def setup_next_animations(self, attribute=None, duration=1000, resolution=17,
                              repeat_behavior=FORWARD, repeat_count=1,
                              end_behavior=controller.HOLD,
                              transformation=LINEAR, end_callback=None):
        """
        Setup the next implicit animations for L{attribute}.

        @param attribute:  attribute for which the animation is setup
        @type attribute:   string
        @param resolution: resolution of the implicit animation
        @type resolution:  float
        @param duration:   duration of the implicit animation
        @type duration:    int

        FIXME: documentation
        """
        if attribute != None:
            ctrl = controller.Controller(duration        = duration,
                                         resolution      = resolution,
                                         repeat_behavior = repeat_behavior,
                                         repeat_count    = repeat_count,
                                         end_behavior    = end_behavior,
                                         transformation  = transformation,
                                         modifiers       = [],
                                         end_callback    = end_callback)
            self._next_controllers[attribute] = ctrl
        else:
            for attribute in self._animated_attributes:
                ctrl = controller.Controller(duration        = duration,
                                             resolution      = resolution,
                                             repeat_behavior = repeat_behavior,
                                             repeat_count    = repeat_count,
                                             end_behavior    = end_behavior,
                                             transformation  = transformation,
                                             modifiers       = [],
                                             end_callback    = end_callback)
                self._next_controllers[attribute] = ctrl

    def update_animation_settings(self, attribute=None, duration=None,
                                  resolution=None, repeat_behavior=None,
                                  repeat_count=None, transformation=None,
                                  end_behavior=None, end_callback=-1):
        """
        Update the animation parameters for one animated attribute (if
        specified) or all of them, if L{attribute} is set to None.

        @keyword attribute:       the property for which we want to update
                                  the settings
        @type attribute:          string or None
        @keyword duration:        new duration of animation, in milliseconds
        @type duration:           int
        @keyword resolution:      new resolution of the animation
        @type resolution:         float
        @keyword repeat_behavior: new repeat behavior of the animation
        @type repeat_behavior:    int
        @keyword repeat_count:    new repeat count of the animation
        @type repeat_count:       int
        @keyword transformation:  new transformation to apply to the animation
        @type transformation:     int
        @keyword end_callback:    new callback to call at the end of animation
        @type end_callback:       callable
        """
        def update_controller(controller):
            if duration != None:
                controller.duration = duration
            if end_behavior != None:
                controller.end_behavior = end_behavior
            if resolution != None:
                controller.resolution = resolution
            if repeat_behavior != None:
                controller.repeat_behavior = repeat_behavior
            if repeat_count != None:
                controller.repeat_count = repeat_count
            if transformation != None:
                controller.transformation = transformation
            if end_callback != -1:
                controller.end_callback = end_callback

        if attribute != None:
            update_controller(self._current_controllers.get(attribute))
        else:
            for controller in self._current_controllers.values():
                update_controller(controller)
            

    def stop_animations(self):
        """
        Stop the animations for all the attributes.
        """
        for controller in self._current_controllers.values():
            controller.stop()
            controller.modifiers = []

    def _animate_attribute_new_animation(self, attribute, target_values):
        # get the current value of the attribute from the proxied object
        current_value = getattr(self._object, attribute)

        # retrieve the currrent controller for the attribute
        ctrl = self._current_controllers[attribute]
        if ctrl.started:
            ctrl.stop()

        # setup timeline and modifier for the new animation
        nb_target_values = len(target_values)
        time_step = 1.0/nb_target_values
        timeline = [keyframe.KeyFrame(target_values[i], time_step*(i+1)) for i in
                    xrange(nb_target_values)]
        timeline.insert(0, keyframe.KeyFrame(current_value, 0.0))

        # check if there is a new controller ready for the attribute
        if self._next_controllers.has_key(attribute):
            # use the new controller
            ctrl = self._next_controllers[attribute]
            self._current_controllers[attribute] = ctrl
            del self._next_controllers[attribute]

        if len(ctrl.modifiers) > 0:
            mod = ctrl.modifiers[0]
            mod.timeline = timeline
        else:
            mod = modifier.Modifier([self._object],
                                    attribute,
                                    timeline)
            ctrl.modifiers = [mod]

        # start the animation
        ctrl.start()


    def _animate_attribute_new_target(self, attribute, target_value):
        # get the current value of the attribute from the proxied object
        current_value = getattr(self._object, attribute)

        # retrieve the current controller for the attribute
        ctrl = self._current_controllers[attribute]
        if ctrl.started:
            # do not do anything if the new target is the same as the old target
            if target_value == ctrl.modifiers[0].timeline[-1].value:
                return
            else:
                # modify the timeline and restart the animation
                ctrl.stop()
                ctrl.modifiers[0].timeline[0].value = current_value
                ctrl.modifiers[0].timeline[-1].value = target_value
                ctrl.start()
        else:
            if target_value != current_value:
                self._animate_attribute_new_animation(attribute, [target_value])


    def _animate_attribute_add_target(self, attribute, target_value):
        # get the current value of the attribute from the proxied object
        current_value = getattr(self._object, attribute)

        # retrieve the current controller for the attribute
        ctrl = self._current_controllers[attribute]
        if ctrl.started:
            # modify the timeline
            time = ctrl._compute_factor()
            #ctrl.stop()
            if time == 0.0:
                i = 1
            else:
                i = ctrl.modifiers[0].next_keyframe_index(time)

            targets = [key.value for key in ctrl.modifiers[0].timeline[i:]]
            targets.append(target_value)

            self._animate_attribute_new_animation(attribute, targets)
            """
            # setup timeline and modifier for the new animation
            nb_targets = len(targets)
            time_step = 1.0/nb_targets
            timeline = [keyframe.KeyFrame(targets[i], time_step*(i+1)) for i in
                        range(nb_targets)]
            timeline.insert(0, keyframe.KeyFrame(current_value, 0.0))
            ctrl.modifiers[0].timeline = timeline

            ctrl.start()
            """
        else:
            self._animate_attribute_new_animation(attribute, [target_value])


    def __setattr__(self, attribute, value):
        if attribute == "_object" or \
           attribute == "_current_controllers" or \
           attribute == "_next_controllers" or \
           attribute == "_animated_attributes" or \
           attribute == "mode":
            # if self._object is set, set it locally
            super(AnimatedObject, self).__setattr__(attribute, value)
        else:
            if attribute in self._animated_attributes:
                # if an animated attribute is to be set, animate it
                if not isinstance(value, list):
                    if self.mode == REPLACE:
                        self._animate_attribute_new_target(attribute, value)
                    elif self.mode == APPEND:
                        self._animate_attribute_add_target(attribute, value)
                else:
                    # FIXME: could also use add_target
                    self._animate_attribute_new_animation(attribute, value)
            else:
                # for any other attribute set, set it to the proxied object
                if isinstance(value, list):
                    value = value[-1]

                setattr(self._object, attribute, value)

    def __getattr__(self, attribute):
        # for any attribute accessed not found locally
        # retrieve it from the proxied object
        if attribute in self._animated_attributes:
            # if an animated attribute is to be retrieved, do not return the
            # real value but the target value
            ctrl = self._current_controllers[attribute]
            if ctrl.started:
                return ctrl.modifiers[0].timeline[-1].value
            else:
                return getattr(self._object, attribute)

        else:
            return getattr(self._object, attribute)

    def is_animated(self, attribute = None):
        """
        Returns True if L{attribute} is currently being animated, False
        otherwise.
        If no attribute is specified, return True if any of the attributes is
        currently animated.

        @param attribute: attribute to check for
        @type attribute:  string
        @rtype: boolean
        """
        if attribute != None:
            controller = self._current_controllers[attribute]
            return controller.started
        else:
            for controller in self._current_controllers.values():
                if controller.started:
                    return True

            return False
